#*******************************************************************************
# Copyright (C) 2007 Novell, Inc. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of Novell, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
#*****************************************************************************/

"""Python Provider for OMC_InitdService

Instruments the CIM class OMC_InitdService

"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2
import os
from subprocess import call, Popen, PIPE
from socket import getfqdn

def _blacklist(file):
    """try to skip the same files that YaST2 does.  See 
    /usr/lib/YaST2/servers_non_y2/ag_initscripts"""

    if file in ('reboot',
                'halt',
                'single',
                'README',
                'core',
                'rc',
                'rx',
                'skeleton',
                'powerfail',
                'boot',
                'boot.orig',
                'Makefile',
                ):
        return True
    for prefix in ('skeleton.',
                   'core.',
                   'core-',
                   'boot.',
                   'halt.',
                   ):
        if file.startswith(prefix):
            return True
    for suffix in ('.new',
                   '.core',
                   '.save',
                   '.old',
                   '.swp',
                   '.rpm',
                   ):
        if file.endswith(suffix):
                return True
    return False
                       

#------------------------------------------------------------------------------
def _get_rpm_install_date(fullpath):
    try:
        l=Popen([r'/bin/rpm', '-qf', '--queryformat', \
                '%{INSTALLTIME}', fullpath], \
                stdout=PIPE).communicate()[0]
        if len(l):
            return pywbem.CIMDateTime.fromtimestamp(long(l))
    except:
        pass
    return None

#------------------------------------------------------------------------------
def _is_auto_start(name):
    try:
        fo = open('/etc/inittab', 'r')
        for line in fo.xreadlines():
            if line.startswith('id:'):
                runlevel = line.split(':')[1]
                fo.close()
                break
        else:
            fo.close()
            return False
        ents = os.listdir('/etc/init.d/rc%s.d' % runlevel)
        for ent in ents:
            if ent.startswith('S') and ent[3:] == name:
                return True
        else:
            return False
    except:
        print '**** Exception caught in _is_auto_start()  !!'
        pass
    return False

class OMC_InitdServiceProvider(CIMProvider2):
    """Instrument the CIM class OMC_InitdService 

    OMC_InitdService represents the function of a system init script found
    in the /etc/init.d directory. The Name field in this class will be the
    name of the script.  The provider of this class knows nothing about
    the details of the service itself.  The methods of this class all
    correspond to actions that can be taken on init.d scripts conforming
    to the LSB spec on system initialization. See
    http://refspecs.freestandards.org/LSB_2.0.1/LSB-Core/LSB-Core/sysinit.html
    for more information.
    
    """

    def get_instance(self, env, model, filtered=False):
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance() for %s' \
                % (self.__class__.__name__, model['name']))
        
        #model['ElementName'] = # TODO (type = unicode) 
        #model['EnabledState'] = # TODO (type = pywbem.Uint16 self.Values.EnabledState) (default=5L)
        #model['HealthState'] = # TODO (type = pywbem.Uint16 self.Values.HealthState) 
        fullpath = '/etc/init.d/' + model['name']
        if not filtered and (_blacklist(model['name']) or 
                not os.path.isfile(fullpath)):
            raise pywbem.CIMError(CIM_ERR_NOT_FOUND)

        fo = open(fullpath, 'r')
        caption = model['name']
        for line in fo.xreadlines():
            if line.startswith('#') and 'Short-Description:' in line:
                caption = line[line.find(':')+1:]
                break
        if caption:
            model['Caption'] = caption
            model['Description'] = caption

        idt = _get_rpm_install_date(fullpath)
        if idt:
            model['InstallDate'] = idt

        osts = []
        cmd = fullpath + ' status'
        st = call(cmd, shell=True)
        if st == self.Values.ServiceStatus.Service_Running:
            osts.append(self.Values.OperationalStatus.OK)
        elif st == self.Values.ServiceStatus.Service_Dead___pid_file_exists:
            osts.append(self.Values.OperationalStatus.Error)
        elif st == self.Values.ServiceStatus.Service_Dead___lock_file_exists:
            osts.append(self.Values.OperationalStatus.Error)
        elif st == self.Values.ServiceStatus.Service_Not_Running:
            osts.append(self.Values.OperationalStatus.Stopped)
        else:
            osts.append(self.Values.OperationalStatus.Unknown)

        if osts:
            model['OperationalStatus'] = osts
                
        is_auto_start = _is_auto_start(model['name'])
        model['StartMode'] = is_auto_start and 'Automatic' or 'Manual'
        model['EnabledDefault'] = is_auto_start and \
                self.Values.EnabledDefault.Enabled or \
                self.Values.EnabledDefault.Disabled

        svcStarted = (st == self.Values.ServiceStatus.Service_Running)
        model['Started'] = svcStarted
        #model['OperationalStatus'] = # TODO (type = [pywbem.Uint16,] self.Values.OperationalStatus) 
        #model['OtherEnabledState'] = # TODO (type = unicode) 
        #model['RequestedState'] = # TODO (type = pywbem.Uint16 self.Values.RequestedState) (default=12L)
        #model['Status'] = # TODO (type = [pywbem.Uint16,]
        #model['StatusDescriptions'] = # TODO (type = [unicode,]) 
        #model['TimeOfLastStateChange'] = # TODO (type = pywbem.CIMDateTime) 
        return model

    def enum_instances(self, env, model, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        model.path.update({'SystemName':None,'SystemCreationClassName':None,
            'CreationClassName':None,'Name':None})
        model['SystemName'] = getfqdn()
        model['SystemCreationClassName'] = 'OMC_ComputerSystem'
        model['CreationClassName'] = 'OMC_InitdService'
        for file in os.listdir('/etc/init.d'):
            fullpath = '/etc/init.d/'+file
            if file[0] == '.' or not os.path.isfile(fullpath):
                continue

            if _blacklist(file):
                continue

            # Key properties    
            model['Name'] = file
            if keys_only:
                yield model
            else:
                try:
                    yield self.get_instance(env, model, filtered=True)
                except pywbem.CIMError, (num, msg):
                    if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                                   pywbem.CIM_ERR_ACCESS_DENIED):
                        raise

    def set_instance(self, env, instance, modify_existing):

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        return instance

    def delete_instance(self, env, instance_name):

        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        # TODO delete the resource
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
    def cim_method_requeststatechange(self, env, object_name, 
                                      param_requestedstate,
                                      param_timeoutperiod):
        """Implements OMC_InitdService.RequestStateChange()

        Requests that the state of the element be changed to the value
        specified in the RequestedState parameter. When the requested
        state change takes place, the EnabledState and RequestedState of
        the element will be the same. Invoking the RequestStateChange
        method multiple times could result in earlier requests being
        overwritten or lost.  If 0 is returned, then the task completed
        successfully and the use of ConcreteJob was not required. If 4096
        (0x1000) is returned, then the task will take some time to
        complete, ConcreteJob will be created, and its reference returned
        in the output parameter Job. Any other return code indicates an
        error condition.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method RequestStateChange() 
            should be invoked.
        param_requestedstate --  The input parameter RequestedState (type pywbem.Uint16 self.Values.RequestStateChange.RequestedState) 
            The state requested for the element. This information will be
            placed into the RequestedState property of the instance if the
            return code of the RequestStateChange method is 0 ('Completed
            with No Error'), 3 ('Timeout'), or 4096 (0x1000) ('Job
            Started'). Refer to the description of the EnabledState and
            RequestedState properties for the detailed explanations of the
            RequestedState values.
            
        param_timeoutperiod --  The input parameter TimeoutPeriod (type pywbem.CIMDateTime) 
            A timeout period that specifies the maximum amount of time that
            the client expects the transition to the new state to take.
            The interval format must be used to specify the TimeoutPeriod.
            A value of 0 or a null parameter indicates that the client has
            no time requirements for the transition.  If this property
            does not contain 0 or null and the implementation does not
            support this parameter, a return code of 'Use Of Timeout
            Parameter Not Supported' must be returned.
            

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.RequestStateChange)
        and a dictionary with the out-parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...)) 
            Reference to the job (can be null if the task is completed).
            

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_requeststatechange()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = []
        #out_params['job'] = # TODO (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))
        rval = None # TODO (type pywbem.Uint32 self.Values.RequestStateChange)
        return (rval, out_params)
        
    def cim_method_reloadservice(self, env, object_name):
        """Implements OMC_InitdService.ReloadService()

        This method causes the configuration of the service to be reloaded
        without actually stopping and restarting the service.  This is an
        optional method in init scripts per LSB spec.  Return values are
        what is returned by the service script, and should follow the LSB
        spec.  See ValueMap.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method ReloadService() 
            should be invoked.

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.ReloadService)
        and a dictionary with the out-parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_reloadservice()' \
                % self.__class__.__name__)

        out_params = []
        cmd = '/etc/init.d/%s reload' % object_name['name']
        rval = pywbem.Uint32(call(cmd, shell=True))
        return (rval, out_params)
        
    def cim_method_stopservice(self, env, object_name):
        """Implements OMC_InitdService.StopService()

        This method stops the service.  Return values are what is returned
        by the service script, and should follow the LSB spec.  See
        ValueMap.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method StopService() 
            should be invoked.
        method -- A pywbem.CIMMethod representing the method meta-data

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.StopService)
        and a dictionary with the out-parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_stopservice()' \
                % self.__class__.__name__)

        out_params = []
        cmd = '/etc/init.d/%s stop' % object_name['name']
        rval = pywbem.Uint32(call(cmd, shell=True))
        return (rval, out_params)
        
    def cim_method_enableservice(self, env, object_name):
        """Implements OMC_InitdService.EnableService()

        This method configures the service's script to be installed or
        enabled in the system's boot (init) script process.  The service
        must already exist on the system and the service's script must
        already exist on the system (usually in the init.d dir).  This
        method adds the service to the runlevels the script is assigned
        to.  See LSB spec on installing init scripts.  
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method EnableService() 
            should be invoked.

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.EnableService)
        and a dictionary with the out-parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_enableservice()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = []
        rval = None # TODO (type pywbem.Uint32 self.Values.EnableService)
        return (rval, out_params)
        
    def cim_method_tryrestartservice(self, env, object_name):
        """Implements OMC_InitdService.TryRestartService()

        This method restarts the service if the service is already running.
        This is an optional method in init scripts per LSB spec.  Return
        values are what is returned by the service script, and should
        follow the LSB spec.  See ValueMap.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method TryRestartService() 
            should be invoked.

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.TryRestartService)
        and a dictionary with the out-parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_tryrestartservice()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = []
        rval = None # TODO (type pywbem.Uint32 self.Values.TryRestartService)
        return (rval, out_params)
        
    def cim_method_restartservice(self, env, object_name):
        """Implements OMC_InitdService.RestartService()

        This method stops and restarts the service if the service is
        already running, otherwise starts the service.  Return values are
        what is returned by the service script, and should follow the LSB
        spec.  See ValueMap.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method RestartService() 
            should be invoked.
        method -- A pywbem.CIMMethod representing the method meta-data

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.RestartService)
        and a dictionary with the out-parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_restartservice()' \
                % self.__class__.__name__)

        out_params = []
        cmd = '/etc/init.d/%s restart' % object_name['name']
        rval = pywbem.Uint32(call(cmd, shell=True))
        return (rval, out_params)
        
    def cim_method_forcereloadservice(self, env, object_name):
        """Implements OMC_InitdService.ForceReloadService()

        This method causes the configuration to be reloaded if the service
        supports this, otherwise restart the service if it is
        runningReturn values are what is returned by the service script,
        and should follow the LSB spec.  See ValueMap.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method ForceReloadService() 
            should be invoked.

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.ForceReloadService)
        and a dictionary with the out-parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_forcereloadservice()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = []
        rval = None # TODO (type pywbem.Uint32 self.Values.ForceReloadService)
        return (rval, out_params)
        
    def cim_method_startservice(self, env, object_name):
        """Implements OMC_InitdService.StartService()

        This method starts the service.  Return values are what is returned
        by the service script, and should follow the LSB spec.  See
        ValueMap.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method StartService() 
            should be invoked.

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.StartService)
        and a dictionary with the out-parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_startservice()' \
                % self.__class__.__name__)

        out_params = []
        cmd = '/etc/init.d/%s start' % object_name['name']
        rval = pywbem.Uint32(call(cmd, shell=True))
        return (rval, out_params)
        
    def cim_method_servicestatus(self, env, object_name):
        """Implements OMC_InitdService.ServiceStatus()

        This method returns the status of the service.  The integer
        returned is what the script returns as its exit value.  See LSB
        spec on init scripts.  Also see ValueMap.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method ServiceStatus() 
            should be invoked.

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.ServiceStatus)
        and a dictionary with the out-parameters

        Output parameters: none
        
        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_servicestatus()' \
                % self.__class__.__name__)

        cmd = '/etc/init.d/%s status > /dev/null 2>&1' % object_name['name']
        rval = pywbem.Uint32(call(cmd, shell=True))
        return (rval, {})
        
    def cim_method_disableservice(self, env, object_name):
        """Implements OMC_InitdService.DisableService()

        This method configures the service's script to be removed or
        disabled in the system's boot (init) script process.  The
        service's script is not removed from the system (init.d dir), nor
        is the service removed from the system.  This method removes the
        service from all runlevels it is currently assigned to.  See LSB
        spec on installing/removing init scripts.  
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method DisableService() 
            should be invoked.

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.DisableService)
        and a dictionary with the out-parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_disableservice()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = []
        rval = None # TODO (type pywbem.Uint32 self.Values.DisableService)
        return (rval, out_params)
        
    class Values(object):
        class Status(object):
            OK = 'OK'
            Error = 'Error'
            Degraded = 'Degraded'
            Unknown = 'Unknown'
            Pred_Fail = 'Pred Fail'
            Starting = 'Starting'
            Stopping = 'Stopping'
            Service = 'Service'
            Stressed = 'Stressed'
            NonRecover = 'NonRecover'
            No_Contact = 'No Contact'
            Lost_Comm = 'Lost Comm'
            Stopped = 'Stopped'

        class RequestedState(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            No_Change = pywbem.Uint16(5)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)
            Not_Applicable = pywbem.Uint16(12)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class HealthState(object):
            Unknown = pywbem.Uint16(0)
            OK = pywbem.Uint16(5)
            Degraded_Warning = pywbem.Uint16(10)
            Minor_failure = pywbem.Uint16(15)
            Major_failure = pywbem.Uint16(20)
            Critical_failure = pywbem.Uint16(25)
            Non_recoverable_error = pywbem.Uint16(30)
            # DMTF_Reserved = ..

        class ReloadService(object):
            Success = pywbem.Uint32(0)
            Generic_Error = pywbem.Uint32(1)
            Invalid_Arguments = pywbem.Uint32(2)
            Unimplemented_Feature = pywbem.Uint32(3)
            Insufficient_Privilege = pywbem.Uint32(4)
            Program_Not_Installed = pywbem.Uint32(5)
            Program_Not_Configured = pywbem.Uint32(6)
            Program_Not_Running = pywbem.Uint32(7)

        class EnabledDefault(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            No_Default = pywbem.Uint16(7)
            Quiesce = pywbem.Uint16(9)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class EnabledState(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shutting_Down = pywbem.Uint16(4)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            In_Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Starting = pywbem.Uint16(10)
            # DMTF_Reserved = 11..32767
            # Vendor_Reserved = 32768..65535

        class EnableService(object):
            Success = pywbem.Uint32(0)
            Failure = pywbem.Uint32(1)

        class StopService(object):
            Success = pywbem.Uint32(0)
            Generic_Error = pywbem.Uint32(1)
            Invalid_Arguments = pywbem.Uint32(2)
            Unimplemented_Feature = pywbem.Uint32(3)
            Insufficient_Privilege = pywbem.Uint32(4)
            Program_Not_Installed = pywbem.Uint32(5)
            Program_Not_Configured = pywbem.Uint32(6)
            Program_Not_Running = pywbem.Uint32(7)

        class StartService(object):
            Success = pywbem.Uint32(0)
            Generic_Error = pywbem.Uint32(1)
            Invalid_Arguments = pywbem.Uint32(2)
            Unimplemented_Feature = pywbem.Uint32(3)
            Insufficient_Privilege = pywbem.Uint32(4)
            Program_Not_Installed = pywbem.Uint32(5)
            Program_Not_Configured = pywbem.Uint32(6)
            Program_Not_Running = pywbem.Uint32(7)

        class TryRestartService(object):
            Success = pywbem.Uint32(0)
            Generic_Error = pywbem.Uint32(1)
            Invalid_Arguments = pywbem.Uint32(2)
            Unimplemented_Feature = pywbem.Uint32(3)
            Insufficient_Privilege = pywbem.Uint32(4)
            Program_Not_Installed = pywbem.Uint32(5)
            Program_Not_Configured = pywbem.Uint32(6)
            Program_Not_Running = pywbem.Uint32(7)

        class OperationalStatus(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            OK = pywbem.Uint16(2)
            Degraded = pywbem.Uint16(3)
            Stressed = pywbem.Uint16(4)
            Predictive_Failure = pywbem.Uint16(5)
            Error = pywbem.Uint16(6)
            Non_Recoverable_Error = pywbem.Uint16(7)
            Starting = pywbem.Uint16(8)
            Stopping = pywbem.Uint16(9)
            Stopped = pywbem.Uint16(10)
            In_Service = pywbem.Uint16(11)
            No_Contact = pywbem.Uint16(12)
            Lost_Communication = pywbem.Uint16(13)
            Aborted = pywbem.Uint16(14)
            Dormant = pywbem.Uint16(15)
            Supporting_Entity_in_Error = pywbem.Uint16(16)
            Completed = pywbem.Uint16(17)
            Power_Mode = pywbem.Uint16(18)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class RestartService(object):
            Success = pywbem.Uint32(0)
            Generic_Error = pywbem.Uint32(1)
            Invalid_Arguments = pywbem.Uint32(2)
            Unimplemented_Feature = pywbem.Uint32(3)
            Insufficient_Privilege = pywbem.Uint32(4)
            Program_Not_Installed = pywbem.Uint32(5)
            Program_Not_Configured = pywbem.Uint32(6)
            Program_Not_Running = pywbem.Uint32(7)

        class RequestStateChange(object):
            Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown_or_Unspecified_Error = pywbem.Uint32(2)
            Cannot_complete_within_Timeout_Period = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Invalid_State_Transition = pywbem.Uint32(4097)
            Use_of_Timeout_Parameter_Not_Supported = pywbem.Uint32(4098)
            Busy = pywbem.Uint32(4099)
            # Method_Reserved = 4100..32767
            # Vendor_Specific = 32768..65535
            class RequestedState(object):
                Enabled = pywbem.Uint16(2)
                Disabled = pywbem.Uint16(3)
                Shut_Down = pywbem.Uint16(4)
                Offline = pywbem.Uint16(6)
                Test = pywbem.Uint16(7)
                Defer = pywbem.Uint16(8)
                Quiesce = pywbem.Uint16(9)
                Reboot = pywbem.Uint16(10)
                Reset = pywbem.Uint16(11)
                # DMTF_Reserved = ..
                # Vendor_Reserved = 32768..65535

        class ForceReloadService(object):
            Success = pywbem.Uint32(0)
            Generic_Error = pywbem.Uint32(1)
            Invalid_Arguments = pywbem.Uint32(2)
            Unimplemented_Feature = pywbem.Uint32(3)
            Insufficient_Privilege = pywbem.Uint32(4)
            Program_Not_Installed = pywbem.Uint32(5)
            Program_Not_Configured = pywbem.Uint32(6)
            Program_Not_Running = pywbem.Uint32(7)

        class ServiceStatus(object):
            Service_Running = pywbem.Uint32(0)
            Service_Dead___pid_file_exists = pywbem.Uint32(1)
            Service_Dead___lock_file_exists = pywbem.Uint32(2)
            Service_Not_Running = pywbem.Uint32(3)
            Unknown = pywbem.Uint32(4)

        class DisableService(object):
            Success = pywbem.Uint32(0)
            Failure = pywbem.Uint32(1)

## end of class OMC_InitdServiceProvider

def get_providers(env): 
    omc_initdservice_prov = OMC_InitdServiceProvider()  
    return {'OMC_InitdService': omc_initdservice_prov} 
