#*******************************************************************************
# Copyright (C) 2007 Novell, Inc. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of Novell, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
#*****************************************************************************/
"""Python Provider for OMC_OperatingSystem

Instruments the CIM class OMC_OperatingSystem

"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2
from socket import getfqdn
import resource
import os,sys
import time
from subprocess import Popen, PIPE
import traceback

def _get_max_number_of_processes():
    cc = 0
    try:
        f = open('/proc/sys/fs/file-max')
        cc = int(f.readline())
    except:
        pass
    return cc

def get_os_path(ns):
    return pywbem.CIMInstanceName(classname='OMC_OperatingSystem', 
            namespace=ns,
            keybindings={'CSCreationClassName': 'OMC_ComputerSystem',
                         'CreationClassName': 'OMC_OperatingSystem',
                         'Name': 'Linux',
                         'CSName': getfqdn()})

class OMC_OperatingSystemProvider(CIMProvider2):
    """Instrument the CIM class OMC_OperatingSystem 

    OMC Operating System
    """

    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        
        meminfo = {}
        fo = open('/proc/meminfo', 'r')
        for line in fo.readlines():
            sline = line.split(':')
            meminfo[sline[0].strip()] = int(sline[1].split()[0])
        fo.close()

        #model.update(TotalSwapSpaceSize = pywbem.Uint64(meminfo['SwapTotal']))
        model['TotalSwapSpaceSize'] = pywbem.CIMProperty('TotalSwapSpaceSize',
                type='uint64', value=None)  # not distinguished from page files.
        model.update(FreePhysicalMemory = pywbem.Uint64(meminfo['MemFree'] +
                    meminfo['Buffers'] + meminfo['Cached']))
        model.update(FreeVirtualMemory = pywbem.Uint64(meminfo['MemFree'] +
                    meminfo['Buffers'] + meminfo['Cached'] + 
                    meminfo['SwapFree']))
        model.update(TotalVisibleMemorySize = pywbem.Uint64(meminfo['MemTotal']))
        model.update(TotalVirtualMemorySize = pywbem.Uint64(meminfo['MemTotal'] +
                    meminfo['SwapTotal']))
        model.update(FreeSpaceInPagingFiles = pywbem.Uint64(meminfo['SwapFree']))
        model.update(SizeStoredInPagingFiles = pywbem.Uint64(meminfo['SwapTotal']))
        
        fnames = os.listdir('/proc')
        numprocs = len([x for x in fnames if x.isdigit()])
        model.update(NumberOfProcesses = pywbem.Uint32(numprocs))
        model.update(CurrentTimeZone = pywbem.Sint16(pywbem.CIMDateTime.get_local_utcoffset()))
        model.update(LocalDateTime = pywbem.CIMDateTime.now())
        fo = open('/proc/uptime', 'r')

        model.update(LastBootUpTime = pywbem.CIMDateTime.fromtimestamp(
            time.time() - float(fo.read().split()[0])))
        fo.close()
        if os.path.exists('/etc/SuSE-release'):
            fo = open('/etc/SuSE-release', 'r')
            version = fo.readline().strip()
            model.update(Version = version)
            model.update(ElementName = version)
            model.update(Description = version)
            model.update(Caption = version)
            fo.close()
            install_time = Popen(['/bin/rpm', '-qf','/etc/SuSE-release',
                '--queryformat','%{INSTALLTIME}'], 
                stdout=PIPE).communicate()[0]
            model.update(InstallDate = pywbem.CIMDateTime.fromtimestamp(int(install_time)))
        # TODO non-SUSE distros
        
        model.update(Distributed = True)
        #model['EnabledDefault'] = # TODO (type = pywbem.Uint16 self.Values.EnabledDefault) (default=2L)
        model.update(EnabledState = self.Values.EnabledState.Enabled)
        model.update(HealthState = self.Values.HealthState.OK)
        model['MaxNumberOfProcesses'] = pywbem.Uint32(_get_max_number_of_processes())

        wk = resource.getrlimit(resource.RLIMIT_DATA)[1]
        if int(wk) < 0:
            wk = 0
        model.update(MaxProcessMemorySize = pywbem.Uint64(int(wk)/1024))

        wk = resource.getrlimit(resource.RLIMIT_NPROC)[1]
        if int(wk) < 0:
            wk = 0
        model.update(MaxProcessesPerUser = pywbem.Uint32(wk))
        model.update(NumberOfLicensedUsers = pywbem.Uint32(0))

        try:
            l = Popen([r'/usr/bin/who','-u'], stdout=PIPE).communicate()[0]
            model.update(NumberOfUsers = pywbem.Uint32(len(l.split('\n'))))
        except:
            model.update(NumberOfUsers = pywbem.Uint32(0))

        model.update(OperationalStatus = [self.Values.OperationalStatus.OK])
        model.update(OSType = self.Values.OSType.LINUX)
        #model['OtherEnabledState'] = # TODO (type = unicode) 
        #model['OtherTypeDescription'] = # TODO (type = unicode) 
        #model['RequestedState'] = # TODO (type = pywbem.Uint16 self.Values.RequestedState) (default=12L)
        #model['Status'] = # TODO (type = unicode self.Values.Status) 
        #model['StatusDescriptions'] = # TODO (type = [unicode,]) 
        #model['TimeOfLastStateChange'] = # TODO (type = pywbem.CIMDateTime) 
        return model

    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        # Key properties    
        model.path.update({'CSCreationClassName':None,
                'CreationClassName':None,
                'Name':None,
                'CSName':None})

        model['CSCreationClassName'] = 'OMC_ComputerSystem'
        model['CreationClassName'] = 'OMC_OperatingSystem'    
        model['Name'] = 'Linux'
        model['CSName'] = getfqdn()
        if keys_only:
            yield model
        else:
            try:
                yield self.get_instance(env, model)
            except pywbem.CIMError, (num, msg):
                if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                               pywbem.CIM_ERR_ACCESS_DENIED):
                    raise

    def set_instance(self, env, instance, previous_instance):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    def cim_method_requeststatechange(self, env, object_name, 
                                      param_requestedstate,
                                      param_timeoutperiod):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_requeststatechange()' \
                % self.__class__.__name__)
        if os.geteuid() != 0:
            raise pywbem.CIMError(pywbem.CIM_ERR_ACCESS_DENIED)
        rstate = int(param_requestedstate)
        if rstate == 3:     # Disable
            logger.log_info('OS Disabled State Requested. Shutting Down now...')
            if os.system('/sbin/shutdown -h now'):
                return (pywbem.Uint32(4), [])
            return (pywbem.Uint32(0), [])
        elif rstate == 4:   # Shutdown
            logger.log_info('OS Shutdown State Requested. Shutting Down now...')
            if os.system('/sbin/shutdown -h now'):
                return (pywbem.Uint32(4), [])
            return (pywbem.Uint32(0), [])
        elif rstate == 6:   # Offline
            logger.log_info('OS Offline State Requested. Shutting Down now...')
            if os.system('/sbin/shutdown -h now'):
                return (pywbem.Uint32(4), [])
            return (pywbem.Uint32(0), [])
        elif rstate == 10:  # Reboot
            logger.log_info('OS Reboot State Requested. Rebooting now...')
            if os.system('/sbin/shutdown -r now'):
                return (pywbem.Uint32(4), [])
            return (pywbem.Uint32(0), [])
        elif rstate == 11:  # Reset
            logger.log_info('OS Reset State Requested. Rebooting now...')
            if os.system('/sbin/shutdown -r now'):
                return (pywbem.Uint32(4), [])
            return (pywbem.Uint32(0), [])
        # Return 'not supported'
        return (pywbem.Uint32(1), [])
        
    def cim_method_reboot(self, env, object_name):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_reboot()' \
                % self.__class__.__name__)
        if os.geteuid() != 0:
            raise pywbem.CIMError(pywbem.CIM_ERR_ACCESS_DENIED)
        logger.log_info('OMC_OperatingSystem.reboot called. '
            'Rebooting system now')
        if os.system('/sbin/shutdown -r now'):
            return ('Fail', [])
        return ('Success', [])
        
    def cim_method_shutdown(self, env, object_name, method):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_shutdown()' \
                % self.__class__.__name__)
        if os.geteuid() != 0:
            raise pywbem.CIMError(pywbem.CIM_ERR_ACCESS_DENIED)
        logger.log_info('OMC_OperatingSystem.shutdown called. '
            'shutdown system now')
        if os.system('/sbin/shutdown -h now'):
            return ('Fail', [])
        return ('Success', [])
        
    class Values(object):
        class Status(object):
            OK = 'OK'
            Error = 'Error'
            Degraded = 'Degraded'
            Unknown = 'Unknown'
            Pred_Fail = 'Pred Fail'
            Starting = 'Starting'
            Stopping = 'Stopping'
            Service = 'Service'
            Stressed = 'Stressed'
            NonRecover = 'NonRecover'
            No_Contact = 'No Contact'
            Lost_Comm = 'Lost Comm'
            Stopped = 'Stopped'

        class RequestedState(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            No_Change = pywbem.Uint16(5)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)
            Not_Applicable = pywbem.Uint16(12)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class HealthState(object):
            Unknown = pywbem.Uint16(0)
            OK = pywbem.Uint16(5)
            Degraded_Warning = pywbem.Uint16(10)
            Minor_failure = pywbem.Uint16(15)
            Major_failure = pywbem.Uint16(20)
            Critical_failure = pywbem.Uint16(25)
            Non_recoverable_error = pywbem.Uint16(30)
            # DMTF_Reserved = ..

        class EnabledDefault(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            No_Default = pywbem.Uint16(7)
            Quiesce = pywbem.Uint16(9)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class EnabledState(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shutting_Down = pywbem.Uint16(4)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            In_Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Starting = pywbem.Uint16(10)
            # DMTF_Reserved = 11..32767
            # Vendor_Reserved = 32768..65535

        class OperationalStatus(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            OK = pywbem.Uint16(2)
            Degraded = pywbem.Uint16(3)
            Stressed = pywbem.Uint16(4)
            Predictive_Failure = pywbem.Uint16(5)
            Error = pywbem.Uint16(6)
            Non_Recoverable_Error = pywbem.Uint16(7)
            Starting = pywbem.Uint16(8)
            Stopping = pywbem.Uint16(9)
            Stopped = pywbem.Uint16(10)
            In_Service = pywbem.Uint16(11)
            No_Contact = pywbem.Uint16(12)
            Lost_Communication = pywbem.Uint16(13)
            Aborted = pywbem.Uint16(14)
            Dormant = pywbem.Uint16(15)
            Supporting_Entity_in_Error = pywbem.Uint16(16)
            Completed = pywbem.Uint16(17)
            Power_Mode = pywbem.Uint16(18)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class RequestStateChange(object):
            Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown_or_Unspecified_Error = pywbem.Uint32(2)
            Cannot_complete_within_Timeout_Period = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Invalid_State_Transition = pywbem.Uint32(4097)
            Use_of_Timeout_Parameter_Not_Supported = pywbem.Uint32(4098)
            Busy = pywbem.Uint32(4099)
            # Method_Reserved = 4100..32767
            # Vendor_Specific = 32768..65535
            class RequestedState(object):
                Enabled = pywbem.Uint16(2)
                Disabled = pywbem.Uint16(3)
                Shut_Down = pywbem.Uint16(4)
                Offline = pywbem.Uint16(6)
                Test = pywbem.Uint16(7)
                Defer = pywbem.Uint16(8)
                Quiesce = pywbem.Uint16(9)
                Reboot = pywbem.Uint16(10)
                Reset = pywbem.Uint16(11)
                # DMTF_Reserved = ..
                # Vendor_Reserved = 32768..65535

        class OSType(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            MACOS = pywbem.Uint16(2)
            ATTUNIX = pywbem.Uint16(3)
            DGUX = pywbem.Uint16(4)
            DECNT = pywbem.Uint16(5)
            Tru64_UNIX = pywbem.Uint16(6)
            OpenVMS = pywbem.Uint16(7)
            HPUX = pywbem.Uint16(8)
            AIX = pywbem.Uint16(9)
            MVS = pywbem.Uint16(10)
            OS400 = pywbem.Uint16(11)
            OS_2 = pywbem.Uint16(12)
            JavaVM = pywbem.Uint16(13)
            MSDOS = pywbem.Uint16(14)
            WIN3x = pywbem.Uint16(15)
            WIN95 = pywbem.Uint16(16)
            WIN98 = pywbem.Uint16(17)
            WINNT = pywbem.Uint16(18)
            WINCE = pywbem.Uint16(19)
            NCR3000 = pywbem.Uint16(20)
            NetWare = pywbem.Uint16(21)
            OSF = pywbem.Uint16(22)
            DC_OS = pywbem.Uint16(23)
            Reliant_UNIX = pywbem.Uint16(24)
            SCO_UnixWare = pywbem.Uint16(25)
            SCO_OpenServer = pywbem.Uint16(26)
            Sequent = pywbem.Uint16(27)
            IRIX = pywbem.Uint16(28)
            Solaris = pywbem.Uint16(29)
            SunOS = pywbem.Uint16(30)
            U6000 = pywbem.Uint16(31)
            ASERIES = pywbem.Uint16(32)
            HP_NonStop_OS = pywbem.Uint16(33)
            HP_NonStop_OSS = pywbem.Uint16(34)
            BS2000 = pywbem.Uint16(35)
            LINUX = pywbem.Uint16(36)
            Lynx = pywbem.Uint16(37)
            XENIX = pywbem.Uint16(38)
            VM = pywbem.Uint16(39)
            Interactive_UNIX = pywbem.Uint16(40)
            BSDUNIX = pywbem.Uint16(41)
            FreeBSD = pywbem.Uint16(42)
            NetBSD = pywbem.Uint16(43)
            GNU_Hurd = pywbem.Uint16(44)
            OS9 = pywbem.Uint16(45)
            MACH_Kernel = pywbem.Uint16(46)
            Inferno = pywbem.Uint16(47)
            QNX = pywbem.Uint16(48)
            EPOC = pywbem.Uint16(49)
            IxWorks = pywbem.Uint16(50)
            VxWorks = pywbem.Uint16(51)
            MiNT = pywbem.Uint16(52)
            BeOS = pywbem.Uint16(53)
            HP_MPE = pywbem.Uint16(54)
            NextStep = pywbem.Uint16(55)
            PalmPilot = pywbem.Uint16(56)
            Rhapsody = pywbem.Uint16(57)
            Windows_2000 = pywbem.Uint16(58)
            Dedicated = pywbem.Uint16(59)
            OS_390 = pywbem.Uint16(60)
            VSE = pywbem.Uint16(61)
            TPF = pywbem.Uint16(62)
            Windows__R__Me = pywbem.Uint16(63)
            Caldera_Open_UNIX = pywbem.Uint16(64)
            OpenBSD = pywbem.Uint16(65)
            Not_Applicable = pywbem.Uint16(66)
            Windows_XP = pywbem.Uint16(67)
            z_OS = pywbem.Uint16(68)
            Microsoft_Windows_Server_2003 = pywbem.Uint16(69)
            Microsoft_Windows_Server_2003_64_Bit = pywbem.Uint16(70)

## end of class OMC_OperatingSystemProvider

class OMC_InstalledOSProvider(CIMProvider2):
    """Instrument the CIM class OMC_InstalledOS 

    A link between the ComputerSystem and the OperatingSystem(s) installed
    or loaded on it. An OperatingSystem is 'installed' on a
    ComputerSystem, when placed on one of its StorageExtents - for
    example, copied to a disk drive or downloaded to Memory. Using this
    definition, an OperatingSystem is 'installed' on a NetPC when
    downloaded via the network.
    
    """

    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        
        #model['PrimaryOS'] = # TODO (type = bool) 
        return model

    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        os = pywbem.CIMInstanceName('OMC_OperatingSystem', 
                namespace=model.path.namespace, 
                keybindings={'CSCreationClassName':'OMC_ComputerSystem',
                             'CreationClassName':'OMC_OperatingSystem',
                             'Name':'Linux',
                             'CSName':getfqdn()})
        cs = pywbem.CIMInstanceName('OMC_ComputerSystem', 
                namespace=model.path.namespace, 
                keybindings={'CreationClassName':'OMC_ComputerSystem',
                             'Name':getfqdn()})

        if model.classname.lower() == 'omc_installedos':
            model.path.update({'GroupComponent':None,'PartComponent':None})
            model['GroupComponent'] = cs
            model['PartComponent'] = os
        else:
            model.path.update({'Dependent':None,'Antecedent':None})
            model['Dependent'] = cs
            model['Antecedent'] = os
        if keys_only:
            yield model
        else:
            try:
                yield self.get_instance(env, model)
            except pywbem.CIMError, (num, msg):
                if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                               pywbem.CIM_ERR_ACCESS_DENIED):
                    raise

    def set_instance(self, env, instance, previous_instance):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    def references(self, env, object_name, model, 
                   result_class_name, role, result_role, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.references()' \
                % self.__class__.__name__)

        if object_name.classname.lower() == 'omc_computersystem' or \
                object_name.classname.lower() == 'omc_operatingsystem':
            return self.simple_refs(env, object_name, model, 
                    result_class_name, role, result_role, keys_only)

## end of class OMC_InstalledOSProvider

def get_providers(env): 
    omc_operatingsystem_prov = OMC_OperatingSystemProvider()  
    omc_installedos_prov = OMC_InstalledOSProvider()
    return {'OMC_OperatingSystem': omc_operatingsystem_prov,
            'OMC_InstalledOS': omc_installedos_prov,
            'OMC_RunningOS': omc_installedos_prov} 
