"""Python Provider for OMC_SyslogNGRecordLog

Instruments the CIM class OMC_SyslogNGRecordLog

"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2
from socket import getfqdn
import os

_sl_list = None
_sl_mtime = None

def _get_syslog_list():
    global _sl_list
    global _sl_mtime
    def partition(s, seq):
        """ S.partition(sep) -> (head, sep, tail)

        Searches for the separator sep in S, and returns the part before it,
        the separator itself, and the part after it.  If the separator is not
        found, returns S and two empty strings.
        """
        try:
            return s.partition(seq)
        except AttributeError:
            try:
                idx = s.index(seq)
            except ValueError:
                return (s, '', '')
            return (s[:idx], seq, s[idx+len(seq):])

    sl_config_file = '/etc/syslog-ng/syslog-ng.conf'
    st = os.stat(sl_config_file)
    if _sl_list and _sl_mtime == st.st_mtime:
        return _sl_list
    _sl_mtime = st.st_mtime
    _sl_list = []
    fo = open(sl_config_file, 'r')
    state = 'i'
    for line in fo:
        line = line.strip()
        if line and line[0] == '#':
            continue
        if state == 'i' and line.startswith('destination'):
            state = 'd'
        if state == 'd':
            head, sep, tail = partition(line, 'file')
            if sep:
                state = 'f'
                line = tail
        if state == 'f':
            head, sep, tail = partition(line, '"')
            if sep:
                line = tail
                head, sep, tail = partition(line, '"')
                if sep:
                    line = head
                    state = 'i'
                    if line.startswith('/dev'):
                        continue # skip /dev/tty10 and /dev/xconsole
                    _sl_list.append(line)
    fo.close()
    return _sl_list



class OMC_SyslogNGRecordLogProvider(CIMProvider2):
    """Instrument the CIM class OMC_SyslogNGRecordLog 

    Class representing a syslog-ng log file
    
    """

    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))
        # If you will be filtering instances yourself according to 
        # property_list, role, result_role, and result_class_name 
        # parameters, set self.filter_results to False
        # self.filter_results = False

    def get_instance(self, env, model):
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)

        ux = model.update

        try:
            id = model['InstanceID']
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        if not id.startswith('OMCSyslogNGRecordLog:'):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        file = id[id.index(':')+1:]
        if file not in _get_syslog_list():
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)

        ux(ElementName=file)
        ux(Name=file)
        ux(MaxNumberOfRecords=pywbem.Uint64(0))
        try:
            fo = open(file, 'r')
            numlines = 0
            while True:
                try:
                    fo.next()
                    numlines+= 1
                except StopIteration:
                    break
            fo.close()
        except:
            numlines = 0
        ux(CurrentNumberOfRecords=pywbem.Uint64(numlines))


        #ux(AvailableRequestedStates=[self.Values.AvailableRequestedStates.<VAL>,]) # TODO 
        #ux(Caption='') # TODO 
        #ux(CommunicationStatus=self.Values.CommunicationStatus.<VAL>) # TODO 
        #ux(Description='') # TODO 
        #ux(DetailedStatus=self.Values.DetailedStatus.<VAL>) # TODO 
        #ux(EnabledDefault=self.Values.EnabledDefault.Enabled) # TODO 
        #ux(EnabledState=self.Values.EnabledState.Not_Applicable) # TODO 
        #ux(HealthState=self.Values.HealthState.<VAL>) # TODO 
        #ux(InstallDate=pywbem.CIMDateTime()) # TODO 
        #ux(LogState=self.Values.LogState.Not_Applicable) # TODO 
        #ux(OperatingStatus=self.Values.OperatingStatus.<VAL>) # TODO 
        #ux(OperationalStatus=[self.Values.OperationalStatus.<VAL>,]) # TODO 
        #ux(OtherEnabledState='') # TODO 
        #ux(OverwritePolicy=self.Values.OverwritePolicy.Unknown) # TODO 
        #ux(PrimaryStatus=self.Values.PrimaryStatus.<VAL>) # TODO 
        #ux(RequestedState=self.Values.RequestedState.Not_Applicable) # TODO 
        #ux(Status=self.Values.Status.<VAL>) # TODO 
        #ux(StatusDescriptions=['',]) # TODO 
        #ux(TimeOfLastStateChange=pywbem.CIMDateTime()) # TODO 
        #ux(TransitioningToState=self.Values.TransitioningToState.Not_Applicable) # TODO 
        return model


    def enum_instances(self, env, model, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        model.path.update({'InstanceID':None})
        for fname in _get_syslog_list():
            model['InstanceID'] = 'OMCSyslogNGRecordLog:' + fname
            state = 'i'
            if keys_only:
                yield model
            else:
                try:
                    yield self.get_instance(env, model)
                except pywbem.CIMError, (num, msg):
                    if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                                   pywbem.CIM_ERR_ACCESS_DENIED):
                        raise

    def set_instance(self, env, instance, modify_existing):

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        return instance

    def delete_instance(self, env, instance_name):

        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        # TODO delete the resource
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
    def cim_method_requeststatechange(self, env, object_name,
                                      param_requestedstate,
                                      param_timeoutperiod):
        """Implements OMC_SyslogNGRecordLog.RequestStateChange()

        Requests that the state of the element be changed to the value
        specified in the RequestedState parameter. When the requested
        state change takes place, the EnabledState and RequestedState of
        the element will be the same. Invoking the RequestStateChange
        method multiple times could result in earlier requests being
        overwritten or lost.  If 0 is returned, then the task completed
        successfully and the use of ConcreteJob was not required. If 4096
        (0x1000) is returned, then the task will take some time to
        complete, ConcreteJob will be created, and its reference returned
        in the output parameter Job. Any other return code indicates an
        error condition.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method RequestStateChange() 
            should be invoked.
        param_requestedstate --  The input parameter RequestedState (type pywbem.Uint16 self.Values.RequestStateChange.RequestedState) 
            The state requested for the element. This information will be
            placed into the RequestedState property of the instance if the
            return code of the RequestStateChange method is 0 ('Completed
            with No Error'), 3 ('Timeout'), or 4096 (0x1000) ('Job
            Started'). Refer to the description of the EnabledState and
            RequestedState properties for the detailed explanations of the
            RequestedState values.
            
        param_timeoutperiod --  The input parameter TimeoutPeriod (type pywbem.CIMDateTime) 
            A timeout period that specifies the maximum amount of time that
            the client expects the transition to the new state to take.
            The interval format must be used to specify the TimeoutPeriod.
            A value of 0 or a null parameter indicates that the client has
            no time requirements for the transition.  If this property
            does not contain 0 or null and the implementation does not
            support this parameter, a return code of 'Use Of Timeout
            Parameter Not Supported' must be returned.
            

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.RequestStateChange)
        and a dictionary with the out-parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...)) 
            Reference to the job (can be null if the task is completed).
            

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, 
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not 
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor 
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_requeststatechange()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = []
        #out_params['job'] = # TODO (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))
        rval = None # TODO (type pywbem.Uint32 self.Values.RequestStateChange)
        return (rval, out_params)
        
    def cim_method_clearlog(self, env, object_name):
        """Implements OMC_SyslogNGRecordLog.ClearLog()

        Requests that the Log be cleared of all entries.  The return value
        should be 0 if the request was successfully executed, 1 if the
        request is not supported, and some other value, as indicated by
        the ValueMap/Values qualifiers, if an error occurred.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method ClearLog() 
            should be invoked.

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.ClearLog)
        and a dictionary with the out-parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, 
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not 
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor 
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_clearlog()' \
                % self.__class__.__name__)

        try:
            id = object_name['InstanceID']
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        if not id.startswith('OMCSyslogNGRecordLog:'):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        file = id[id.index(':')+1:]
        if file not in _get_syslog_list():
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        fo = open(file, 'w')
        fo.close()
        out_params = []
        rval = self.Values.ClearLog.Completed_with_no_error
        return (rval, out_params)
        
    class Values(object):
        class DetailedStatus(object):
            Not_Available = pywbem.Uint16(0)
            No_Additional_Information = pywbem.Uint16(1)
            Stressed = pywbem.Uint16(2)
            Predictive_Failure = pywbem.Uint16(3)
            Non_Recoverable_Error = pywbem.Uint16(4)
            Supporting_Entity_in_Error = pywbem.Uint16(5)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class RequestedState(object):
            Unknown = pywbem.Uint16(0)
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            No_Change = pywbem.Uint16(5)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)
            Not_Applicable = pywbem.Uint16(12)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class HealthState(object):
            Unknown = pywbem.Uint16(0)
            OK = pywbem.Uint16(5)
            Degraded_Warning = pywbem.Uint16(10)
            Minor_failure = pywbem.Uint16(15)
            Major_failure = pywbem.Uint16(20)
            Critical_failure = pywbem.Uint16(25)
            Non_recoverable_error = pywbem.Uint16(30)
            # DMTF_Reserved = ..

        class LogState(object):
            Unknown = pywbem.Uint16(0)
            Normal = pywbem.Uint16(2)
            Erasing = pywbem.Uint16(3)
            Not_Applicable = pywbem.Uint16(4)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class TransitioningToState(object):
            Unknown = pywbem.Uint16(0)
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            No_Change = pywbem.Uint16(5)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Defer = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)
            Not_Applicable = pywbem.Uint16(12)

        class EnabledDefault(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            No_Default = pywbem.Uint16(7)
            Quiesce = pywbem.Uint16(9)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class EnabledState(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shutting_Down = pywbem.Uint16(4)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            In_Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Starting = pywbem.Uint16(10)
            # DMTF_Reserved = 11..32767
            # Vendor_Reserved = 32768..65535

        class ClearLog(object):
            Completed_with_no_error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unspecified_Error = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            # DMTF_Reserved = 6..0x0FFF
            # Method_Reserved = 0x1000..0x7FFF
            # Vendor_Reserved = 0x8000..

        class AvailableRequestedStates(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Defer = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)

        class Status(object):
            OK = 'OK'
            Error = 'Error'
            Degraded = 'Degraded'
            Unknown = 'Unknown'
            Pred_Fail = 'Pred Fail'
            Starting = 'Starting'
            Stopping = 'Stopping'
            Service = 'Service'
            Stressed = 'Stressed'
            NonRecover = 'NonRecover'
            No_Contact = 'No Contact'
            Lost_Comm = 'Lost Comm'
            Stopped = 'Stopped'

        class CommunicationStatus(object):
            Unknown = pywbem.Uint16(0)
            Not_Available = pywbem.Uint16(1)
            Communication_OK = pywbem.Uint16(2)
            Lost_Communication = pywbem.Uint16(3)
            No_Contact = pywbem.Uint16(4)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class OperationalStatus(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            OK = pywbem.Uint16(2)
            Degraded = pywbem.Uint16(3)
            Stressed = pywbem.Uint16(4)
            Predictive_Failure = pywbem.Uint16(5)
            Error = pywbem.Uint16(6)
            Non_Recoverable_Error = pywbem.Uint16(7)
            Starting = pywbem.Uint16(8)
            Stopping = pywbem.Uint16(9)
            Stopped = pywbem.Uint16(10)
            In_Service = pywbem.Uint16(11)
            No_Contact = pywbem.Uint16(12)
            Lost_Communication = pywbem.Uint16(13)
            Aborted = pywbem.Uint16(14)
            Dormant = pywbem.Uint16(15)
            Supporting_Entity_in_Error = pywbem.Uint16(16)
            Completed = pywbem.Uint16(17)
            Power_Mode = pywbem.Uint16(18)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class OperatingStatus(object):
            Unknown = pywbem.Uint16(0)
            Not_Available = pywbem.Uint16(1)
            In_Service = pywbem.Uint16(2)
            Starting = pywbem.Uint16(3)
            Stopping = pywbem.Uint16(4)
            Stopped = pywbem.Uint16(5)
            Aborted = pywbem.Uint16(6)
            Dormant = pywbem.Uint16(7)
            Completed = pywbem.Uint16(8)
            Migrating = pywbem.Uint16(9)
            Emigrating = pywbem.Uint16(10)
            Immigrating = pywbem.Uint16(11)
            Snapshotting = pywbem.Uint16(12)
            Shutting_Down = pywbem.Uint16(13)
            In_Test = pywbem.Uint16(14)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class OverwritePolicy(object):
            Unknown = pywbem.Uint16(0)
            Wraps_When_Full = pywbem.Uint16(2)
            Never_Overwrites = pywbem.Uint16(7)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class RequestStateChange(object):
            Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown_or_Unspecified_Error = pywbem.Uint32(2)
            Cannot_complete_within_Timeout_Period = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Invalid_State_Transition = pywbem.Uint32(4097)
            Use_of_Timeout_Parameter_Not_Supported = pywbem.Uint32(4098)
            Busy = pywbem.Uint32(4099)
            # Method_Reserved = 4100..32767
            # Vendor_Specific = 32768..65535
            class RequestedState(object):
                Enabled = pywbem.Uint16(2)
                Disabled = pywbem.Uint16(3)
                Shut_Down = pywbem.Uint16(4)
                Offline = pywbem.Uint16(6)
                Test = pywbem.Uint16(7)
                Defer = pywbem.Uint16(8)
                Quiesce = pywbem.Uint16(9)
                Reboot = pywbem.Uint16(10)
                Reset = pywbem.Uint16(11)
                # DMTF_Reserved = ..
                # Vendor_Reserved = 32768..65535

        class PrimaryStatus(object):
            Unknown = pywbem.Uint16(0)
            OK = pywbem.Uint16(1)
            Degraded = pywbem.Uint16(2)
            Error = pywbem.Uint16(3)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

## end of class OMC_SyslogNGRecordLogProvider

class OMC_SyslogNGLogRecordProvider(CIMProvider2):
    """Instrument the CIM class OMC_SyslogNGLogRecord 

    Class representing a record in a log file.
    
    """

    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))
        # If you will be filtering instances yourself according to 
        # property_list, role, result_role, and result_class_name 
        # parameters, set self.filter_results to False
        # self.filter_results = False

    def get_instance(self, env, model, line=None):
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        
        ux = model.update

        # TODO fetch system resource matching the following keys:
        #   model['LogCreationClassName']
        #   model['MessageTimestamp']
        #   model['RecordID']
        #   model['LogName']
        #   model['CreationClassName']
        try:
            id = model['LogName']
            if not id.startswith('OMCSyslogNGRecordLog:'):
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
            fname = id[id.index(':')+1:]
            lineno = int(model['RecordID'])
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        except ValueError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)

        if line is None:
            fo = open(fname, 'r')
            cnt = 0
            for line in fo:
                cnt+= 1
                if cnt == lineno:
                    break
            else:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        ux(RecordData=line.strip().decode('latin_1', 'xmlcharrefreplace'))
        #ux(RecordData=line.strip())
        ux(ElementName=fname)

        #ux(Caption='') # TODO 
        #ux(DataFormat='') # TODO 
        #ux(Description='') # TODO 
        #ux(Locale='') # TODO 
        #ux(RecordFormat='') # TODO 
        return model

    def enum_instances(self, env, model, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        model.path.update({'LogCreationClassName':None,
            'CreationClassName':None})
        model['LogCreationClassName'] = 'OMC_SyslogNGRecordLog'
        model['CreationClassName'] = 'OMC_SyslogNGLogRecord'
        for fname in _get_syslog_list():
            try:
                fo = open(fname, 'r')
            except IOError:
                continue
            model['LogName'] = 'OMCSyslogNGRecordLog:' + fname
            lineno = 0
            for line in fo:
                #model['MessageTimestamp'] = pywbem.CIMDateTime.now()
                model['MessageTimestamp'] = pywbem.CIMDateTime('99990101000000.000000+000')
                lineno+= 1
                model['RecordID'] = str(lineno)
                if keys_only:
                    yield model
                else:
                    try:
                        yield self.get_instance(env, model, line)
                    except pywbem.CIMError, (num, msg):
                        if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                                       pywbem.CIM_ERR_ACCESS_DENIED):
                            raise

    def set_instance(self, env, instance, modify_existing):

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        return instance

    def delete_instance(self, env, instance_name):

        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        # TODO delete the resource
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
## end of class OMC_SyslogNGLogRecordProvider
class OMC_SyslogNGLogManagesRecordProvider(CIMProvider2):
    """Instrument the CIM class OMC_SyslogNGLogManagesRecord 

    Association between a RecordLog and its LogRecords.
    
    """

    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))
        # If you will be filtering instances yourself according to 
        # property_list, role, result_role, and result_class_name 
        # parameters, set self.filter_results to False
        # self.filter_results = False

    def get_instance(self, env, model):
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        
        ux = model.update

        # TODO fetch system resource matching the following keys:
        #   model['Record']
        #   model['Log']

        return model

    def enum_instances(self, env, model, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        while False: # TODO more instances?
            # TODO fetch system resource
            # Key properties    
            #model['Record'] = # TODO (type = REF (pywbem.CIMInstanceName(classname='OMC_SyslogNGLogRecord', ...))    
            #model['Log'] = # TODO (type = REF (pywbem.CIMInstanceName(classname='OMC_SyslogNGRecordLog', ...))
            if keys_only:
                yield model
            else:
                try:
                    yield self.get_instance(env, model)
                except pywbem.CIMError, (num, msg):
                    if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                                   pywbem.CIM_ERR_ACCESS_DENIED):
                        raise

    def set_instance(self, env, instance, modify_existing):

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        return instance

    def delete_instance(self, env, instance_name):

        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        # TODO delete the resource
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
    def references(self, env, object_name, model, 
                   result_class_name, role, result_role, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.references()' \
                % self.__class__.__name__)
        
        model.path.update({'Record':None,'Log':None})

        ch = env.get_cimom_handle()
        if (not role or role.lower() == 'record') and \
           ch.is_subclass(object_name.namespace, 
                       sub=object_name.classname, 
                       super='OMC_SyslogNGLogRecord'):
            model['Record'] = object_name
            logfilename = ''.join(object_name['LogName'].split(':')[1:])
            if logfilename in _get_syslog_list():
                model['Log'] = pywbem.CIMInstanceName('OMC_SyslogNGRecordLog',
                        keybindings={'InstanceID':'OMCSyslogNGRecordLog:' + \
                                logfilename})
                yield model 

        if (not role or role.lower() == 'log') and \
           ch.is_subclass(object_name.namespace, 
                       sub=object_name.classname, 
                       super='OMC_SyslogNGRecordLog'):
            model['Log'] = object_name
            id = object_name['InstanceID']
            if not id.startswith('OMCSyslogNGRecordLog:'):
                return
            file = id[id.index(':')+1:]
            try:
                fo = open(file, 'r')
            except:
                return
            recordname = pywbem.CIMInstanceName('OMC_SyslogNGLogRecord', 
                    keybindings={'LogCreationClassName':'OMC_SyslogNGRecordLog',
                        'MessageTimestamp':pywbem.CIMDateTime('99990101000000.000000+000'),
                        'LogName':id,
                        'CreationClassName':'OMC_SyslogNGLogRecord'})
            cnt = 0
            model['Record'] = recordname
            while True:
                try:
                    fo.next()
                    cnt+= 1
                    recordname['RecordID'] = str(cnt)
                    yield model 
                except StopIteration:
                    break
            fo.close()
## end of class OMC_SyslogNGLogManagesRecordProvider
                       
class OMC_SyslogNGUseOfLogProvider(CIMProvider2):

    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))
        # If you will be filtering instances yourself according to 
        # property_list, role, result_role, and result_class_name 
        # parameters, set self.filter_results to False
        # self.filter_results = False

    def get_instance(self, env, model):
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        
        ux = model.update

        # TODO fetch system resource matching the following keys:
        #   model['Dependent']
        #   model['Antecedent']

        #ux(RecordedData='') # TODO 
        return model

    def enum_instances(self, env, model, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        model.path.update({'Dependent':None,
            'Antecedent':None})
        for log in _get_syslog_list():
            model['Dependent'] = pywbem.CIMInstanceName('OMC_ComputerSystem', 
                    keybindings={'CreationClassName':'OMC_ComputerSystem',
                                 'Name':getfqdn()},
                    namespace=model.path.namespace)
            model['Antecedent'] = pywbem.CIMInstanceName('OMC_SyslogNGRecordLog', 
                    keybindings={'InstanceID':'OMCSyslogNGRecordLog:'+log},
                    namespace=model.path.namespace)
            if keys_only:
                yield model
            else:
                try:
                    yield self.get_instance(env, model)
                except pywbem.CIMError, (num, msg):
                    if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                                   pywbem.CIM_ERR_ACCESS_DENIED):
                        raise

    def set_instance(self, env, instance, modify_existing):

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        return instance

    def delete_instance(self, env, instance_name):

        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        # TODO delete the resource
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
    def references(self, env, object_name, model, 
                   result_class_name, role, result_role, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.references()' \
                % self.__class__.__name__)

        model.path.update({'Dependent':None,'Antecedent':None})

        ch = env.get_cimom_handle()
        if (not role or role.lower() == 'dependent') and \
                object_name.classname.lower() == 'omc_computersystem':
            model['Dependent'] = object_name
            for log in _get_syslog_list():
                model['Antecedent'] = pywbem.CIMInstanceName('OMC_SyslogNGRecordLog', 
                        keybindings={'InstanceID':'OMCSyslogNGRecordLog:'+log},
                        namespace=object_name.namespace)
                yield model 

        if (not role or role.lower() == 'antecedent') and \
                object_name.classname.lower() == 'omc_syslogngrecordlog':
            model['Antecedent'] = object_name
            model['Dependent'] = pywbem.CIMInstanceName('OMC_ComputerSystem',
                    keybindings={'CreationClassName':'OMC_ComputerSystem',
                                 'Name':getfqdn()}, 
                    namespace=object_name.namespace)
            yield model 

## end of class OMC_SyslogNGUseOfLogProvider
class OMC_SyslogNGRecordLogCapabilitiesProvider(CIMProvider2):
    """Instrument the CIM class OMC_SyslogNGRecordLogCapabilities 

    EnabledLogicalElementCapabilities for SyslogNGRecordLog.  This class is
    here for conformance with the SMASH Record Log Profile.  For other
    details, see parent class.
    
    """

    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))
        # If you will be filtering instances yourself according to 
        # property_list, role, result_role, and result_class_name 
        # parameters, set self.filter_results to False
        # self.filter_results = False

    def get_instance(self, env, model):
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        
        ux = model.update

        try:
            id = model['InstanceID']
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        if not id.startswith('OMCSyslogNGRecordLog:'):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        file = id[id.index(':')+1:]
        if file not in _get_syslog_list():
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)

        ux(ElementNameEditSupported=False)
        ux(ElementName=file)
        #ux(Caption='') # TODO 
        #ux(Description='') # TODO 
        #ux(MaxElementNameLen=pywbem.Uint16()) # TODO 
        #ux(RequestedStatesSupported=[self.Values.RequestedStatesSupported.<VAL>,]) # TODO 
        return model

    def enum_instances(self, env, model, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        model.path.update({'InstanceID':None})
        for log in _get_syslog_list():
            model['InstanceID'] = 'OMCSyslogNGRecordLog:'+log
            if keys_only:
                yield model
            else:
                try:
                    yield self.get_instance(env, model)
                except pywbem.CIMError, (num, msg):
                    if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                                   pywbem.CIM_ERR_ACCESS_DENIED):
                        raise

    def set_instance(self, env, instance, modify_existing):

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        return instance

    def delete_instance(self, env, instance_name):

        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        # TODO delete the resource
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
    class Values(object):
        class RequestedStatesSupported(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Defer = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)

## end of class OMC_SyslogNGRecordLogCapabilitiesProvider
class OMC_SyslogNGRecordLogToCapabilitiesProvider(CIMProvider2):
    """Instrument the CIM class OMC_SyslogNGRecordLogToCapabilities 

    Associates OMC_SyslogNGRecordLog to  OMC_SyslogNGRecordLogCapabilities.
    
    """

    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))
        # If you will be filtering instances yourself according to 
        # property_list, role, result_role, and result_class_name 
        # parameters, set self.filter_results to False
        # self.filter_results = False

    def get_instance(self, env, model):
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        
        ux = model.update

        # TODO fetch system resource matching the following keys:
        #   model['Capabilities']
        #   model['ManagedElement']

        return model

    def enum_instances(self, env, model, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        model.path.update({'Capabilities':None, 'ManagedElement':None})

        logname = pywbem.CIMInstanceName('OMC_SyslogNGRecordLog', 
                keybindings={},
                namespace=model.path.namespace)
        capname = pywbem.CIMInstanceName('OMC_SyslogNGRecordLogCapabilities',
                keybindings={},
                namespace=model.path.namespace)
        model['Capabilities'] = capname
        model['ManagedElement'] = logname
        for log in _get_syslog_list():
            id = 'OMCSyslogNGRecordLog:' + log
            capname['InstanceID'] = id
            logname['InstanceID'] = id
            if keys_only:
                yield model
            else:
                try:
                    yield self.get_instance(env, model)
                except pywbem.CIMError, (num, msg):
                    if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                                   pywbem.CIM_ERR_ACCESS_DENIED):
                        raise

    def set_instance(self, env, instance, modify_existing):

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        return instance

    def delete_instance(self, env, instance_name):

        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        # TODO delete the resource
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
    def references(self, env, object_name, model, 
                   result_class_name, role, result_role, keys_only):

        logger = env.get_logger()
        logger.log_debug('Entering %s.references()' \
                % self.__class__.__name__)
        ch = env.get_cimom_handle()

        model.path.update({'Capabilities':None,'ManagedElement':None})

        try:
            id = object_name['InstanceID']
        except KeyError:
            return
        if not id.startswith('OMCSyslogNGRecordLog:'):
            return
        file = id[id.index(':')+1:]
        if file not in _get_syslog_list():
            return
        if (not role or role.lower() == 'capabilities') and \
                object_name.classname.lower() == 'omc_syslogngrecordlogcapabilities':
            model['Capabilities'] = object_name
            model['ManagedElement'] = pywbem.CIMInstanceName('OMC_SyslogNGRecordLog', 
                    keybindings={'InstanceID':id}, 
                    namespace=object_name.namespace)
            yield model 

        if (not role or role.lower() == 'managedelement') and \
                object_name.classname.lower() == 'omc_syslogngrecordlog':
            model['ManagedElement'] = object_name
            model['Capabilities'] = pywbem.CIMInstanceName('OMC_SyslogNGRecordLogCapabilities', 
                    keybindings={'InstanceID':id}, 
                    namespace=object_name.namespace)
            yield model 

## end of class OMC_SyslogNGRecordLogToCapabilitiesProvider



def get_providers(env): 
    omc_syslogngrecordlog_prov = OMC_SyslogNGRecordLogProvider(env)  
    omc_syslognglogrecord_prov = OMC_SyslogNGLogRecordProvider(env)  
    omc_syslognglogmanagesrecord_prov = OMC_SyslogNGLogManagesRecordProvider(env) 
    omc_syslognguseoflog_prov = OMC_SyslogNGUseOfLogProvider(env)
    omc_syslogngrecordlogcapabilities_prov = OMC_SyslogNGRecordLogCapabilitiesProvider(env)
    omc_syslogngrecordlogtocapabilities_prov = OMC_SyslogNGRecordLogToCapabilitiesProvider(env)
    return {'OMC_SyslogNGRecordLog': omc_syslogngrecordlog_prov,
            'OMC_SyslogNGLogRecord': omc_syslognglogrecord_prov,
            'OMC_SyslogNGLogManagesRecord': omc_syslognglogmanagesrecord_prov,
            'OMC_SyslogNGUseOfLog': omc_syslognguseoflog_prov,
            'OMC_SyslogNGRecordLogCapabilities': omc_syslogngrecordlogcapabilities_prov,
            'OMC_SyslogNGRecordLogToCapabilities': omc_syslogngrecordlogtocapabilities_prov} 

