import os
import socket
import tempfile
from subprocess import Popen, PIPE


import pywbem
from pywbem.cim_provider2 import CIMProvider2

_fqdn = socket.getfqdn()

ntp_inst_date = None
try:
    _install_time = Popen(['rpm', '-qf','/etc/ntp.conf',
            '--queryformat','%{INSTALLTIME}'],
            stdout=PIPE).communicate()[0]
    ntp_inst_date = pywbem.CIMDateTime.fromtimestamp(int(_install_time))
except:
    ntp_inst_date = None
    pass

###############################################################################
def _isIPV4Address(addr):
    try:
        mm = socket.inet_aton(addr)
        return True
    except socket.error:
        return False

###############################################################################
class NTPServer(object):
    ###########################################################################
    def __init__(self):
        self.server_name = ''
        self.prefer = False
        self.key = 0
        self.ntp_version = 0
        self.min_poll = 0
        self.max_poll = 0

    ###########################################################################
    def config_line(self):
        line = 'server %s' % self.server_name
        if self.prefer:
            line += ' prefer'
        if self.key:
            line += ' key %s' % str(self.key)
        if self.ntp_version:
            line += ' version %s' % str(self.ntp_version)
        if self.min_poll:
            line += ' minpoll %s' % str(self.min_poll)
        if self.max_poll:
            line += ' maxpoll %s' % str(self.max_poll)
        line += '\n'
        return line

    ###########################################################################
    @classmethod
    def __server_from_conf_line(cls, line):
        tl = line.strip()
        if len(tl) < 6:
            return None
        if tl[0:6].lower() != 'server':
            return None
        toks = tl.split()
        tokslen = len(toks)
        if tokslen < 2:
            return None
        server = NTPServer()
        server.server_name = toks[1]
        if tokslen < 3:
            return server
        i = 2
        while i < tokslen:
            utok = toks[i].lower()
            if utok == 'prefer':
                server.prefer = True
                i += 1
                continue
            if i < (tokslen-1):
                if utok == 'key':
                    i += 1
                    server.key = int(toks[i])
                elif utok == 'version':
                    i += 1
                    server.ntp_version = int(toks[i])
                elif utok == 'mode':
                    i += 1
                    # Ignore
                elif utok == 'minpoll':
                    i += 1
                    server.min_poll = int(toks[i])
                elif utok == 'maxpoll':
                    i += 1
                    server.max_poll = int(toks[i])
            i += 1

            
        return server

    def fill_cim_obj(self, model, keys_only=False):
        global ntp_inst_date
        model['SystemName'] = _fqdn
        model['SystemCreationClassName'] = 'OMC_ComputerSystem'
        model['CreationClassName'] = 'OMC_RemoteTimeServicePort'
        model['Name'] = self.server_name
        if keys_only:
            return model 
        model['AccessInfo'] = self.server_name
        model['InfoFormat'] = _isIPV4Address(self.server_name) \
                and pywbem.Uint16(3) or pywbem.Uint16(2)
        model['PortProtocol'] = pywbem.Uint16(2)
        model['EnabledState'] = pywbem.Uint16(2)
        model['RequestedState'] = pywbem.Uint16(12)
        model['EnabledDefault'] = pywbem.Uint16(2)
        model['InstallDate'] = ntp_inst_date
        model['OperationalStatus'] = [pywbem.Uint16(2)]
        model['Status'] = 'OK'
        model['HealthState'] = pywbem.Uint16(5)
        model['StartMode'] = 'Automatic'
        model['Prefer'] = self.prefer and True or False
        if self.key:
            model['Key'] = pywbem.Uint32(self.key)
        if self.ntp_version:
            model['ntpversion'] = pywbem.Uint32(self.ntp_version)
        if self.min_poll:
            model['minpoll'] = pywbem.Uint32(self.min_poll)
        if self.max_poll:
            model['maxpoll'] = pywbem.Uint32(self.max_poll)
        return model
    
    ###########################################################################
    @classmethod
    def get_servers(cls):
        servers = []
        if not os.path.exists('/etc/ntp.conf'):
            return None
        f = open('/etc/ntp.conf')
        try:
            for line in f:
                server = NTPServer.__server_from_conf_line(line)
                if server is not None:
                    servers.append(server)
            return servers
        finally:
            f.close()

    ###########################################################################
    @classmethod
    def get_server(cls, server_name):
        if not os.path.exists('/etc/ntp.conf'):
            return None
        f = open('/etc/ntp.conf')
        try:
            usn = server_name.lower()
            for line in f:
                server = NTPServer.__server_from_conf_line(line)
                if server is not None:
                    if server.server_name.lower() == usn:
                        return server
            return None
        finally:
            f.close()

    ###########################################################################
    @classmethod
    def delete_server(cls, server_name):
        modified = False
        usname = server_name.lower()
        tfd,tfname = tempfile.mkstemp(dir='/etc')
        tfile = os.fdopen(tfd, 'w')
        cfile = open('/etc/ntp.conf')
        try:
            for line in cfile:
                server = NTPServer.__server_from_conf_line(line)
                if server is None:
                    tfile.write(line)
                    continue
                if server.server_name.lower() == usname:
                    modified = True
                else:
                    tfile.write(line)
            tfile.close()
            cfile.close()
            if not modified:
                os.unlink(tfname)
            else:
                os.unlink('/etc/ntp.conf')
                os.rename(tfname, '/etc/ntp.conf')
                os.chmod('/etc/ntp.conf', 0644)
            return modified
        except Exception,einst:
            tfile.close()
            cfile.close()
            if tfname:
                try:
                    os.unlink(tfname)
                except:
                    pass
            raise einst

    ###########################################################################
    @classmethod
    def set_server(cls, ntpserver):
        modified = False
        server_written = False
        usname = ntpserver.server_name.lower()
        tfd,tfname = tempfile.mkstemp(dir='/etc')
        tfile = os.fdopen(tfd, 'w')
        cfile = open('/etc/ntp.conf')
        try:
            for line in cfile:
                server = NTPServer.__server_from_conf_line(line)
                if server is None:
                    tfile.write(line)
                elif server.server_name.lower() == usname:
                    modified = True
                    server_written = True
                    tfile.write(ntpserver.config_line())
                else:
                    if ntpserver.prefer and server.prefer:
                        modified = True
                        server.prefer = False
                        tfile.write(ntpserver.config_line())
                    else:
                        tfile.write(line)
            if not server_written:
                modified = True
                tfile.write(ntpserver.config_line())
            tfile.close()
            cfile.close()
            if not modified:
                os.unlink(tfname)
            else:
                os.unlink('/etc/ntp.conf')
                os.rename(tfname, '/etc/ntp.conf')
                os.chmod('/etc/ntp.conf', 0644)
            return modified
        except Exception,einst:
            tfile.close()
            cfile.close()
            if tfname:
                try:
                    os.unlink(tfname)
                except:
                    pass
            raise einst

###############################################################################
def _clock_conf_key_value(line):
    tline = line.strip(' \n\r')
    if not len(tline) or tline.startswith('#'):
        return None
    ndx = tline.find('=')
    if ndx != -1:
        key = tline[0:ndx].strip()
        value = tline[ndx+1:].strip(' "')
        return (key,value)
    return None

###############################################################################
def _clock_conf_get_time_zone():
    is_utc = False
    timezone = ''
    cnt = 0
    try:
        f = open('/etc/sysconfig/clock')
    except:
        return None

    try:
        for line in f:
            kv = _clock_conf_key_value(line)
            if kv is not None:
                if kv[0].lower() == 'hwclock':
                    is_utc = (kv[1].lower() == '-u')
                    cnt += 1
                elif kv[0].lower() == 'timezone':
                    timezone = kv[1]
                    cnt += 1
            if cnt > 1:
                break
        return (is_utc, timezone)
    finally:
        f.close()

###############################################################################
def _clock_conf_modify(is_utc, timezone):
    modified = False
    tfname = None
    utc_val = is_utc and '"-u"' or '"--localtime"'
    tfd,tfname = tempfile.mkstemp(dir='/etc/sysconfig')
    tfile = os.fdopen(tfd, 'w')
    cfile = open('/etc/sysconfig/clock')
    try:
        for line in cfile:
            kv = _clock_conf_key_value(line)
            if kv is None:
                tfile.write(line)
                continue
            if kv[0].lower() == 'hwclock':
                tfile.write('HWCLOCK=%s\n' % utc_val)
                modified = True
            elif kv[0].lower() == 'timezone':
                tfile.write('TIMEZONE=%s\n' % timezone)
                modified = True
            else:
                tfile.write(line)
        tfile.close()
        cfile.close()
        if not modified:
            os.unlink(tfname)
        else:
            os.unlink('/etc/sysconfig/clock')
            os.rename(tfname, '/etc/sysconfig/clock')
            os.chmod('/etc/sysconfig/clock', 0644)
    except Exception,einst:
        tfile.close()
        cfile.close()
        if tfname:
            try:
                os.unlink(tfname)
            except:
                pass
        raise einst

###############################################################################
def _is_cs_ref(model):
    if 'CreationClassName' not in model \
            or 'Name' not in model:
        return False
    if model['CreationClassName'].lower() != 'omc_computersystem':
        return False
    return model['Name'].lower() == _fqdn.lower()

###############################################################################
def _is_service_ref(model):
    for nm in ['SystemCreationClassName', 'SystemName', 'CreationClassName', 'Name']:
        if nm not in model:
            return False
    if model['SystemCreationClassName'].lower() != 'omc_computersystem':
        return False
    if model['SystemName'].lower() != _fqdn.lower():
        return False
    if model['CreationClassName'].lower() != 'omc_systemtimeservice':
        return False
    return model['Name'].lower() == 'timeservice'

###############################################################################
def _fill_service_instance(model, keys_only):
    model['SystemCreationClassName'] = 'OMC_ComputerSystem'
    model['SystemName'] = _fqdn
    model['CreationClassName'] = 'OMC_SystemTimeService'
    model['Name'] = 'timeservice'
    if not keys_only:
        model['EnabledState'] = pywbem.Uint16(2)
        model['RequestedState'] = pywbem.Uint16(12)
        model['EnabledDefault'] = pywbem.Uint16(2)
        model['TimeOfLastStateChange'] = pywbem.CIMDateTime(
            '00000000000000.000000:000')
        model['Started'] = True
        model['InstallDate'] = pywbem.CIMDateTime(
            '00000000000000.000000:000')
        model['OperationalStatus'] = [pywbem.Uint16(2)]
        model['Status'] = 'OK'
        model['HealthState'] = pywbem.Uint16(5)
        model['StartMode'] = 'Automatic'
    return model

###############################################################################
class OMC_SystemTimeServiceProvider(CIMProvider2):
    """Instrument the CIM class OMC_SystemTimeService"""

    ###########################################################################
    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))
        # If you will be filtering instances yourself according to 
        # property_list, role, result_role, and result_class_name 
        # parameters, set self.filter_results to False
        # self.filter_results = False

    ###########################################################################
    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        if not _is_service_ref(model):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        return _fill_service_instance(model, False)

    ###########################################################################
    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
        model.path.update({
                'Name':None,
                'CreationClassName':None,
                'SystemName':None,
                'SystemCreationClassName':None,
                })
        yield _fill_service_instance(model, keys_only)

    ###########################################################################
    def set_instance(self, env, instance, previous_instance):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    ###########################################################################
    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    ###########################################################################
    def cim_method_requeststatechange(self, env, object_name, 
            param_requestedstate, param_timeoutperiod):
        # Not supported
        return (pywbem.Uint32(1), {})
        
    ###########################################################################
    def cim_method_startservice(self, env, object_name):
        # Not supported
        return (pywbem.Uint32(1), {})
        
    ###########################################################################
    def cim_method_stopservice(self, env, object_name):
        # Not supported
        return (pywbem.Uint32(1), {})
        
    ###########################################################################
    def cim_method_managetime(self, env, object_name,
            param_timedata, param_getrequest, param_managedelement):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_managetime()' \
                % self.__class__.__name__)
        if not _is_service_ref(object_name):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        if param_getrequest:    # Is this a 'get' request?
            return (pywbem.Uint32(0),
                {'TimeData':pywbem.CIMDateTime.now()})

        # At this point we know it is a 'set' request
        if os.geteuid() != 0:
            raise pywbem.CIMError(pywbem.CIM_ERR_ACCESS_DENIED,
                'You must have admin privileges on this machine '
                'in order to set the time')

        if not param_timedata:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "'TimeData' parameter is required for a set operation")
        if param_timedata.is_interval:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "'TimeData' parameter must be a timestamp and not an "
                "interval")
        new_time = param_timedata.datetime.ctime()

        ccval = _clock_conf_get_time_zone()
        if ccval is not None:
            is_utc, timezone = ccval
        else:
            is_utc = False

        try:
            # Set the system time
            dt = param_timedata.datetime
            dtstr = '%02d%02d%02d%02d%04d' % \
                    (dt.month, dt.day, dt.hour, dt.minute, dt.year)
            cc = os.system('/bin/date %s' % dtstr)
            if cc:
                raise Exception('Error returned from date command: %d' \
                    % int(cc))
        except Exception,inst:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                'Exception encountered trying to set hardware clock: %s' \
                    % str(inst))
        try:
            # Set the hardware clock
            cmd = '/sbin/hwclock --systohc '
            cmd += is_utc and '-u' or '--localtime'
            cc = os.system(cmd)
            if cc:
                raise Exception('Error returned from hwclock: %d' \
                    % int(cc))
        except Exception,inst:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                'Exception encountered syncing hardware clock with system ' \
                'time' % str(inst))
        return (pywbem.Uint32(0), {})

## end of class OMC_SystemTimeServiceProvider

###############################################################################
class OMC_HostedTimeServiceProvider(CIMProvider2):
    """Instrument the CIM class OMC_HostedTimeService"""

    ###########################################################################
    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))

    ###########################################################################
    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        for nm in ['Dependent', 'Antecedent']:
            if nm not in model:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        if not _is_service_ref(model['Dependent']):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        if not _is_cs_ref(model['Antecedent']):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        return model

    ###########################################################################
    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
        model.path.update({'Antecedent':None,'Dependent':None})
        model['Antecedent'] = pywbem.CIMInstanceName(
            classname='OMC_ComputerSystem',
            namespace=model.path.namespace,
            keybindings={'CreationClassName':'OMC_ComputerSystem',
                'Name':_fqdn})
        ref = pywbem.CIMInstanceName(classname='OMC_SystemTimeService',
                namespace=model.path.namespace)
        model['Dependent'] = _fill_service_instance(ref, True)
        yield model

    ###########################################################################
    def set_instance(self, env, instance, previous_instance):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    ###########################################################################
    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    ###########################################################################
    def references(self, env, object_name, model, 
                   result_class_name, role, result_role, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.references()' \
                % self.__class__.__name__)
        ch = env.get_cimom_handle()
        model.path.update({'Antecedent':None, 'Dependent':None})
        if object_name.classname.lower() == 'omc_systemtimeservice':
            if not _is_service_ref(object_name):
                return
            if role and role.lower() != 'dependent':
                return
            if result_role and result_role.lower() != 'antecedent':
                return
            if result_class_name:
                if not ch.is_subclass(
                        object_name.namespace, sub='omc_computersystem',
                        super=result_class_name):
                    return
            model['Dependent'] = object_name
            model['Antecedent'] = pywbem.CIMInstanceName(
                classname='OMC_ComputerSystem',
                namespace=object_name.namespace,
                keybindings={'CreationClassName':'OMC_ComputerSystem',
                    'Name':_fqdn})
            yield model

        elif object_name.classname.lower() == 'omc_computersystem':
            if not _is_cs_ref(object_name):
                return
            if role and role.lower() != 'antecedent':
                return
            if result_role and result_role.lower() != 'dependent':
                return
            if result_class_name:
                if not ch.is_subclass(
                        object_name.namespace, sub='omc_systemtimeservice',
                        super=result_class_name):
                    return
            model['Antecedent'] = object_name
            ref = pywbem.CIMInstanceName(classname='OMC_SystemTimeService',
                    namespace=object_name.namespace)
            model['Dependent'] = _fill_service_instance(ref, True)
            yield model

## end of class OMC_HostedTimeServiceProvider

###############################################################################
class OMC_RemoteTimeServicePortProvider(CIMProvider2):
    """Instrument the CIM class OMC_RemoteTimeServicePort """

    ###########################################################################
    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))

    ###########################################################################
    @classmethod
    def __get_ntp_install_date(cls):
        try:
            install_time = Popen(['rpm', '-qf','/etc/ntp.conf',
                    '--queryformat','%{INSTALLTIME}'],
                    stdout=PIPE).communicate()[0]
            return pywbem.CIMDateTime.fromtimestamp(int(install_time))
        except:
            return None

    ###########################################################################
    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        try:
            server_name = model['Name']
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        svr = NTPServer.get_server(server_name)
        if not svr:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        return svr.fill_cim_obj(model, False)

    ###########################################################################
    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
        servers = NTPServer.get_servers()
        model.path.update({
                'CreationClassName':None,
                'Name':None,
                'SystemName':None,
                'SystemCreationClassName':None,
                })
        for svr in servers:
            yield svr.fill_cim_obj(model, keys_only)

    ###########################################################################
    def set_instance(self, env, instance, previous_instance):
        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        if os.geteuid() != 0:
            raise pywbem.CIMError(pywbem.CIM_ERR_ACCESS_DENIED)
        try:
            sname = previous_instance and previous_instance['Name'] or instance['Name']
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "'Name' is a required property")
        server = NTPServer.get_server(sname)
        if previous_instance:
            if not server:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
            for k,v in instance.properties.items():
                previous_instance[k] = v
            instance = previous_instance
        else:
            if server:
                raise pywbem.CIMError(pywbem.CIM_ERR_ALREADY_EXISTS)
            server = NTPServer()
            server.server_name = sname

        if 'prefer' in instance and instance['prefer']:
            server.prefer = True
        else:
            server.prefer = False
        if 'key' in instance and instance['key']:
            server.key = int(instance['key'])
        else:
            server.key = 0

        if 'ntpversion' in instance and instance['ntpversion']:
            v = int(instance['ntpversion'])
            if v < 0 or v > 4:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    'Invalid NTPVersion specified')
            server.ntp_version = v
        else:
            server.ntp_version = 0
        if 'minpoll' in instance and instance['minpoll']:
            server.min_poll = int(instance['minpoll'])
        else:
            server.min_poll = 0
        if 'maxpoll' in instance and instance['maxpoll']:
            server.max_poll = int(instance['maxpoll'])
        else:
            server.max_poll = 0
        NTPServer.set_server(server)
        return instance

    ###########################################################################
    def delete_instance(self, env, instance_name):
        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)
        if os.geteuid() != 0:
            raise pywbem.CIMError(pywbem.CIM_ERR_ACCESS_DENIED)
        try:
            sname = instance_name['Name']
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        server = NTPServer.get_server(sname)
        if not server:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        NTPServer.delete_server(sname)
        
    ###########################################################################
    def cim_method_requeststatechange(self, env, object_name, 
                                      param_requestedstate,
                                      param_timeoutperiod):
        # Not supported
        return (pywbem.Uint32(1), {})

## end of class OMC_RemoteTimeServicePortProvider

###############################################################################
class OMC_TimeServiceAccessBySAPProvider(CIMProvider2):
    """Instrument the CIM class OMC_TimeServiceAccessBySAP"""

    ###########################################################################
    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))

    ###########################################################################
    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        try:
            antref = model['Antecedent']
            depref = model['Dependent']
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        # Verify service ref
        if not _is_service_ref(antref):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)

        # Verify SAP ref
        try:
            server_name = depref['Name']
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        svr = NTPServer.get_server(server_name)
        if not svr:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        return model

    ###########################################################################
    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        model.path.update({'Antecedent':None, 'Dependent':None})

        ref = pywbem.CIMInstanceName(
            classname='OMC_SystemTimeService',
            namespace=model.path.namespace);
        model['Antecedent'] = _fill_service_instance(ref, True)
        ref = pywbem.CIMInstanceName(
            classname='OMC_RemoteTimeServicePort',
            namespace=model.path.namespace);
        servers = NTPServer.get_servers()
        for svr in servers:
            model['Dependent'] = svr.fill_cim_obj(ref, True)
            yield model

    ###########################################################################
    def set_instance(self, env, instance, previous_instance):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    ###########################################################################
    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    ###########################################################################
    def references(self, env, object_name, model, 
                   result_class_name, role, result_role, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.references()' \
                % self.__class__.__name__)
        ch = env.get_cimom_handle()
        model.path.update({'antecedent':None,'dependent':None})
        if object_name.classname.lower() == 'omc_systemtimeservice':
            if not _is_service_ref(object_name):
                return
            if role and role.lower() != 'antecedent':
                return
            if result_role and result_role.lower() != 'dependent':
                return
            if result_class_name and len(result_class_name):
                if not ch.is_subclass(object_name.namespace, 
                        sub='OMC_RemoteTimeServicePort',
                        super=result_class_name):
                    return
            model['Antecedent'] = object_name
            ref = pywbem.CIMInstanceName(
                classname='OMC_RemoteTimeServicePort',
                namespace=object_name.namespace);
            servers = NTPServer.get_servers()
            for svr in servers:
                model['Dependent'] = svr.fill_cim_obj(ref, True)
                yield model
        elif object_name.classname.lower() == 'omc_remotetimeserviceport':
            if role and role.lower() != 'dependent':
                return
            if result_role and result_role.lower() != 'antecedent':
                return
            if result_class_name and len(result_class_name):
                if not ch.is_subclass(object_name.namespace, 
                        sub='OMC_SystemTimeService',
                        super=result_class_name):
                    return
            try:
                server_name = object_name['Name']
            except KeyError:
                return
            svr = NTPServer.get_server(server_name)
            if not svr:
                return
            model['Dependent'] = object_name
            ref = pywbem.CIMInstanceName(
                classname='OMC_SystemTimeService',
                namespace=object_name.namespace);
            model['Antecedent'] = _fill_service_instance(ref, True)
            yield model
## end of class OMC_TimeServiceAccessBySAPProvider

###############################################################################
class OMC_TimeZoneSettingDataProvider(CIMProvider2):
    """Instrument the CIM class OMC_TimeZoneSettingData"""

    ###########################################################################
    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))

    ###########################################################################
    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        try:
            iid = model['InstanceID']
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        if iid.lower() != 'omc:timezone':
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        tzcc = _clock_conf_get_time_zone()
        if not tzcc:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        model['ElementName'] = 'omc:timezone'
        model['UTCHardwareClock'] = tzcc[0] 
        model['TimeZone'] = tzcc[1]
        model['Caption'] = 'Time Zone Setting'
        model['Description'] = 'Local time zone for the system'
        return model

    ###########################################################################
    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
        tzcc = _clock_conf_get_time_zone()
        if not tzcc:
            return
        model.path.update({'InstanceID':None})
        model['InstanceID'] = 'omc:timezone'
        if not keys_only:
            model['ElementName'] = 'omc:timezone'
            model['UTCHardwareClock'] = tzcc[0] 
            model['TimeZone'] = tzcc[1]
            model['Caption'] = 'Time Zone Setting'
            model['Description'] = 'Local time zone for the system'
        yield model

    ###########################################################################
    def set_instance(self, env, instance, previous_instance):
        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        if os.geteuid() != 0:
            raise pywbem.CIMError(pywbem.CIM_ERR_ACCESS_DENIED)
        if previous_instance:
            for k,v in instance.properties.items():
                previous_instance[k] = v
            instance = previous_instance
        try:
            iid = instance['InstanceID']
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        if iid.lower() != 'omc:timezone':
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)

        if 'timezone' not in instance or not instance['timezone']:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "'TimeZone' is a required property")
        if 'utchardwareclock' not in instance:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "'UTCHardwareClass' is a required property")
        time_zone = instance['timezone']
        is_utc = instance['utchardwareclock']
        tzcc = _clock_conf_get_time_zone()
        if not tzcc:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        old_time_zone = tzcc[1]
        # Verify zone info
        zone_file = '/usr/share/zoneinfo/%s' % time_zone
        if not os.path.exists(zone_file):
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                '%s is not a valid time zone specification' % instance['timezone'])
   
        cmd = '/usr/sbin/zic -l %s' % time_zone
        cc = os.system(cmd)
        if cc:
            try:
                # Try to revert back to old
                os.system('/usr/sbin/zic -l %s' % old_time_zone)
            except:
                pass
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                'Failed to execute command %s' % cmd)

        try:
            _clock_conf_modify(is_utc, time_zone)
        except Exception,einst:
            # Try to recover old
            try:
                os.system('/usr/sbin/zic -l %s' % old_time_zone)
            except:
                pass
            raise einst
        return instance

    ###########################################################################
    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
## end of class OMC_TimeZoneSettingDataProvider

###############################################################################
class OMC_TimeServiceTimeZoneSettingDataProvider(CIMProvider2):
    """Instrument the CIM class OMC_TimeServiceTimeZoneSettingData"""

    ###########################################################################
    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))

    ###########################################################################
    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        try:
            srvref = model['managedelement']
            if not _is_service_ref(srvref):
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
            sdref = model['settingdata']
            if sdref['InstanceID'].lower() != 'omc:timezone':
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        model['IsDefault'] = pywbem.Uint16(0)
        model['IsCurrent'] = pywbem.Uint16(1)
        return model

    ###########################################################################
    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
        model.path.update({'settingdata':None,'managedelement':None})
        model['settingdata'] = pywbem.CIMInstanceName(
            classname='OMC_TimeZoneSettingData',
            namespace=model.path.namespace,
            keybindings={'InstanceID':'omc:timezone'})
        ref = pywbem.CIMInstanceName(classname='OMC_SystemTimeService',
            namespace=model.path.namespace)
        model['managedelement'] = _fill_service_instance(ref, True)
        if not keys_only:
            model['IsDefault'] = pywbem.Uint16(0)
            model['IsCurrent'] = pywbem.Uint16(1)
        yield model

    ###########################################################################
    def set_instance(self, env, instance, previous_instance):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    ###########################################################################
    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    ###########################################################################
    def references(self, env, object_name, model, 
                   result_class_name, role, result_role, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.references()' \
                % self.__class__.__name__)
        ch = env.get_cimom_handle()
        model.path.update({'settingdata':None,'managedelement':None})
        if object_name.classname.lower() == 'omc_systemtimeservice':
            if role and role.lower() != 'managedelement':
                return
            if result_role and result_role.lower() != 'settingdata':
                return
            if result_class_name:
                if not ch.is_subclass(object_name.namespace,
                        sub='omc_timezonesettingdata',
                        super=result_class_name):
                    return
            if not _is_service_ref(object_name):
                return
            model['managedelement'] = object_name
            model['settingdata'] = pywbem.CIMInstanceName(
                classname='OMC_TimeZoneSettingData',
                namespace=object_name.namespace,
                keybindings={'InstanceID':'omc:timezone'})
            model['IsDefault'] = pywbem.Uint16(0)
            model['IsCurrent'] = pywbem.Uint16(1)
            yield model
        elif object_name.classname.lower() == 'omc_timezonesettingdata':
            if role and role.lower() != 'settingdata':
                return
            if result_role and result_role.lower() != 'managedelement':
                return
            if result_class_name:
                if not ch.is_subclass(object_name.namespace,
                        sub='omc_systemtimeservice',
                        super=result_class_name):
                    return
            try:
                if object_name['InstanceID'].lower() != 'omc:timezone':
                    return
            except KeyError:
                return
            model['settingdata'] = object_name
            ref = pywbem.CIMInstanceName(classname='OMC_SystemTimeService',
                namespace=object_name.namespace)
            model['managedelement'] = _fill_service_instance(ref, True)
            model['IsDefault'] = pywbem.Uint16(0)
            model['IsCurrent'] = pywbem.Uint16(1)
            yield model

## end of class OMC_TimeServiceTimeZoneSettingDataProvider

###############################################################################
class OMC_TimeServiceAvailableToElementProvider(CIMProvider2):
    """Instrument the CIM class OMC_TimeServiceAvailableToElement"""

    ###########################################################################
    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))

    ###########################################################################
    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        try:
            tsref = model['serviceprovided']
            if not _is_service_ref(tsref):
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
            csref = model['userofservice']
            if not _is_cs_ref(csref):
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        return model

    ###########################################################################
    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
        model.path.update({'userofservice':None,'serviceprovided':None})
        model['userofservice'] = pywbem.CIMInstanceName(
            classname='OMC_ComputerSystem',
            namespace=model.path.namespace,
            keybindings={'CreationClassName':'OMC_ComputerSystem',
                'Name':_fqdn})
        ref = pywbem.CIMInstanceName(classname='OMC_SystemTimeService',
                namespace=model.path.namespace)
        model['serviceprovided'] = _fill_service_instance(ref, True)
        yield model

    ###########################################################################
    def set_instance(self, env, instance, previous_instance):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    ###########################################################################
    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    ###########################################################################
    def references(self, env, object_name, model, 
                   result_class_name, role, result_role, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.references()' \
                % self.__class__.__name__)
        ch = env.get_cimom_handle()
        model.path.update({'userofservice':None,'serviceprovided':None})
        if object_name.classname.lower() == 'omc_systemtimeservice':
            if role and role.lower() != 'serviceprovided':
                return
            if result_role and result_role.lower() != 'userofservice':
                return
            if result_class_name:
                if not ch.is_subclass(object_name.namespace, 
                        sub='omc_computersystem',
                        super=result_class_name):
                    return
            if not _is_service_ref(object_name):
                return
            model['serviceprovided'] = object_name
            model['userofservice'] = pywbem.CIMInstanceName(
                classname='OMC_ComputerSystem',
                namespace=object_name.namespace,
                keybindings={'CreationClassName':'OMC_ComputerSystem',
                    'Name':_fqdn})
            yield model

        elif object_name.classname.lower() == 'omc_computersystem':
            if role and role.lower() != 'userofservice':
                return
            if result_role and result_role.lower() != 'serviceprovided':
                return
            if result_class_name:
                if not ch.is_subclass(object_name.namespace, 
                        sub='omc_systemtimeservice',
                        super=result_class_name):
                    return
            if not _is_cs_ref(object_name):
                return
            model['userofservice'] = object_name
            ref = pywbem.CIMInstanceName(classname='OMC_SystemTimeService',
                    namespace=object_name.namespace)
            model['serviceprovided'] = _fill_service_instance(ref, True)
            yield model

## end of class OMC_TimeServiceAvailableToElementProvider

###############################################################################
def get_providers(env): 
    omc_timeserviceavailabletoelement_prov = OMC_TimeServiceAvailableToElementProvider(env)  
    omc_timeservicetimezonesettingdata_prov = OMC_TimeServiceTimeZoneSettingDataProvider(env)  
    omc_timezonesettingdata_prov = OMC_TimeZoneSettingDataProvider(env)  
    omc_timeserviceaccessbysap_prov = OMC_TimeServiceAccessBySAPProvider(env)  
    omc_remotetimeserviceport_prov = OMC_RemoteTimeServicePortProvider(env)  
    omc_systemtimeservice_prov = OMC_SystemTimeServiceProvider(env)  
    omc_hostedtimeservice_prov = OMC_HostedTimeServiceProvider(env)  
    return {'OMC_SystemTimeService': omc_systemtimeservice_prov,
        'OMC_HostedTimeService': omc_hostedtimeservice_prov,
        'OMC_RemoteTimeServicePort': omc_remotetimeserviceport_prov,
        'OMC_TimeServiceAccessBySAP': omc_timeserviceaccessbysap_prov,
        'OMC_TimeZoneSettingData': omc_timezonesettingdata_prov,
        'OMC_TimeServiceTimeZoneSettingData': omc_timeservicetimezonesettingdata_prov,
        'OMC_TimeServiceAvailableToElement': omc_timeserviceavailabletoelement_prov}

