#*******************************************************************************
# Copyright (C) 2007 Novell, Inc. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  - Neither the name of Novell, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived from this
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
#*****************************************************************************/
"""Python Provider for OMC_UnixProcess

Instruments the CIM class OMC_UnixProcess

"""

import pywbem, os, sys, pwd, signal, time
from socket import getfqdn

import OMC_LogicalFile
from OMC_OperatingSystem import get_os_path

from resource import getpagesize

from pywbem.cim_provider2 import CIMProvider2

_pagesize = getpagesize()

PROC_STATE_DOESNT_EXIST         = -1
PROC_STATE_UNKNOWN              = 0
PROC_STATE_OTHER                = 1
PROC_STATE_READY                = 2
PROC_STATE_RUNNING              = 3
PROC_STATE_BLOCKED              = 4
PROC_STATE_SUSPENDED_BLOCKED    = 5
PROC_STATE_SUSPENDED_READY      = 6
PROC_STATE_TERMINATED           = 7
PROC_STATE_STOPPED              = 8
PROC_STATE_GROWING              = 9

_HERTZ = 100
_system_boot_time = None

###############################################################################
def _set_system_boot_time():
    global _system_boot_time
    _system_boot_time = None
    f = open('/proc/stat')
    try:
        for line in f:
            if line.find('btime') == 0:
                _system_boot_time = long(line.split()[1])
                break
    finally:
        f.close()
    return _system_boot_time


# Set _system_boot_time on import
_set_system_boot_time()

###############################################################################
def _get_proc_stat_field(pid, ndx):
    f = open('/proc/%d/stat' % pid)
    fields = f.readline().split()
    f.close()
    return fields[ndx]

###############################################################################
def _get_proc_stat_fields(pid):
    f = open('/proc/%d/stat' % pid)
    fields = f.readline().split()
    f.close()
    return fields

###############################################################################
def _get_task_state(pid, tid):
    fname = '/proc/%s/task/%s/stat' % (str(pid), str(tid))
    fd = open(fname, 'r')
    fields = fd.readline().split()
    fd.close()
    if fields[2] == 'T':
        cc = 1
    elif fields[2] == 'R':
        cc = 2
    elif fields[2] == 'D':
        cc = 3
    elif fields[2] == 'Z':
        cc = 4
    elif fields[2] == 'S':
        cc = 5
    elif fields[2] == 'W':
        cc = 6
    else:
        cc = 7
    return cc

#####################################
def _stateThruTasks(pid):
    psra = [
        PROC_STATE_UNKNOWN,
        PROC_STATE_STOPPED,
        PROC_STATE_RUNNING,
        PROC_STATE_BLOCKED,
        PROC_STATE_TERMINATED,
        PROC_STATE_SUSPENDED_READY,
        PROC_STATE_OTHER,
        PROC_STATE_UNKNOWN]
    try:
        tnames = os.listdir('/proc/%d/task' % int(pid))
    except:
        return PROC_STATE_DOESNT_EXIST
    savedpriority = 7
    for tname in tnames:
        try:
            tname = str(long(tname))    # verify its numeric
        except:
            continue
        statepriority = _get_task_state(pid, tname)
        if statepriority > -1 and statepriority < savedpriority:
            savedpriority = statepriority
    return psra[savedpriority]

def get_proc_stats(pid):
    sd = {}
    sd['pid'] = pid
    try:
        fields = _get_proc_stat_fields(pid)
        sd['exec_name'] = fields[1][1:len(fields[1])-1]
        sd['state'] = _stateThruTasks(pid)
        sd['parent_process_id'] = long(fields[3])
        sd['process_group'] = long(fields[4])
        sd['session_id'] = long(fields[5])
        sd['tty_number'] = long(fields[6])
        user_mode_time = long(fields[13])
        sd['user_mode_time'] = user_mode_time
        kernel_mode_time = long(fields[14])
        sd['kernel_mode_time'] = kernel_mode_time
        sd['priority'] = long(fields[17])
        sd['nice_value'] = long(fields[18])
        start_time = (long(fields[21]) / 100) + _system_boot_time
        sd['start_time'] = pywbem.CIMDateTime.fromtimestamp(start_time)
        sd['virtual_mem_size'] = long(fields[22])
        tot_time = user_mode_time + kernel_mode_time
        secs_since_boot = time.time() - _system_boot_time
        seconds = secs_since_boot - start_time / _HERTZ
        pcpu = int(0)
        if seconds:
            pcpu = int((tot_time * 1000 / _HERTZ) / seconds)
        if pcpu > 999:
            pcpu = 999
        sd['percent_cpu'] = float(pcpu) / 10.0
    except:
        print '#### Exception caught in get_proc_stats %d' % pid
    try:
        f = open('/proc/%d/statm' % pid)
        fields = f.readline().split()
        f.close()
        sd['resident_set_size'] = pywbem.Uint64(fields[1]) * _pagesize
        sd['shared_mem_size'] = pywbem.Uint64(fields[2]) * _pagesize
    except:
        pass
    return sd

#####################################
class LinuxProcess(object):
    def __init__(self, pid):
        self.pid = int(pid)

    def procs():
        pids = os.listdir('/proc')
        for spid in pids:
            try:
                ipid = int(spid)    # verify numeric
                yield LinuxProcess(ipid)
            except:
                pass    # Not an all numeric file name

    procs = staticmethod(procs)

    def get_proc(pid):
        try:
            ipid = int(pid) # verify numeric
        except:
            return None
        if os.path.isdir('/proc/%d' % ipid):
            return LinuxProcess(ipid)
        return None

    get_proc = staticmethod(get_proc)

    def get_instance_name(self, ns):
        return pywbem.CIMInstanceName(classname='OMC_UnixProcess', namespace=ns,
                keybindings={'OSCreationClassName':'OMC_OperatingSystem',
                             'CreationClassName':'OMC_UnixProcess',
                             'Handle':str(self.pid),
                             'CSCreationClassName':'OMC_ComputerSystem',
                             'OSName':'Linux',
                             'CSName':getfqdn()})
    
    def get_real_uid(self):
        ruid = -1L
        f = open('/proc/%d/status' % self.pid)
        try:
            for line in f:
                if line.find('Uid') == 0:
                    ruid = long(line.split()[1])
                    break
        finally:
            f.close()
        return ruid

    def get_percent_cpu(self, include_children = False):
        try:
            fields =  _get_proc_stat_fields(self.pid)
            utime = long(fields[13])
            stime = long(fields[14])
            cutime = long(fields[15])
            cstime = long(fields[16])
            start_time = long(fields[21])
            tot_time = utime + stime
            if include_children:
                tot_time += cutime + cstime

            secs_since_boot = time.time() - _system_boot_time
            seconds = secs_since_boot - start_time / _HERTZ
            pcpu = 0
            if seconds:
                pcpu = (tot_time * 1000 / _HERTZ) / seconds
            if pcpu > 999:
                pcpu = 999
        except:
            return None
        return float(pcpu) / 10.0

    def kill(self, sig):
        try:
            os.kill(self.pid, sig)
        except OSError:
            return -1
        return 0

    def get_command_line(self):
        try:
            f = open('/proc/%d/cmdline' % self.pid)
            l = f.readline()
            f.close()
            rl = []
            for arg in l.split('\0'):
                if arg:
                    rl.append(arg)
        except:
            return []
        return rl

    def get_env(self):
        try:
            f = open('/proc/%d/environ' % self.pid)
            l = f.readline()
            f.close()
            d = dict(i.split('=',1) for i in l.split('\0')) 
        except:
            return {}
        return d

    def get_open_files(self):
        try:
            rv = []
            dname = '/proc/%d/fd/' % self.pid
            for fname in os.listdir(dname):
                rv.append(os.readlink(dname + fname))
        except:
            return []
        return rv

    def get_root(self):
        try:
            s = os.readlink('/proc/%d/root' % self.pid)
        except:
            return ''
        return s

    def get_cwd(self):
        try:
            s = os.readlink('/proc/%d/cwd' % self.pid)
        except:
            return ''
        return s

    def get_module_path(self):
        try:
            s = os.readlink('/proc/%d/exe' % self.pid)
        except:
            return None
        return s

    def get_exec_name(self):
        try:
            s = _get_proc_stat_field(self.pid, 1)
            s = s[1:len(s)-1]
        except:
            s = ''
        return s

    def get_state(self):
        return _stateThruTasks(self.pid)

    def get_parent_pid(self):
        try:
            s = _get_proc_stat_field(self.pid, 3)
        except:
            return -1
        return int(s)

    def get_process_group(self):
        try:
            s = _get_proc_stat_field(self.pid, 4)
        except:
            return -1
        return int(s)

    def get_tty_number(self):
        try:
            s = _get_proc_stat_field(self.pid, 6)
        except:
            return -1
        return int(s)

    def get_session_id(self):
        try:
            s = _get_proc_stat_field(self.pid, 5)
        except:
            return -1
        return int(s)

    def get_owning_pgrp_of_tty(self):
        try:
            s = _get_proc_stat_field(self.pid, 7)
        except:
            return -1
        return int(s)

    def get_flags(self):
        try:
            s = _get_proc_stat_field(self.pid, 8)
        except:
            return long(-1)
        return long(s)

    def get_priority(self):
        try:
            s = _get_proc_stat_field(self.pid, 17)
        except:
            return long(-1)
        return long(s) - 15
    
    def get_kernel_mode_time(self):
        try:
            s = _get_proc_stat_field(self.pid, 14)
        except:
            return long(-1)
        return long(s) * 10

    def get_user_mode_time(self):
        try:
            s = _get_proc_stat_field(self.pid, 13)
        except:
            return long(-1)
        return long(s) * 10

    def get_nice_value(self):
        try:
            s = _get_proc_stat_field(self.pid, 18)
        except:
            return long(-1)
        return long(s)

    def get_start_time(self):
        try:
            s = _get_proc_stat_field(self.pid, 21)
            st = (long(s) / 100) + _system_boot_time
            start_time = pywbem.CIMDateTime.fromtimestamp(st)
        except:
            return None
        return start_time

    def get_virtual_mem_size(self):
        try:
            s = _get_proc_stat_field(self.pid, 22)
        except:
            return long(-1)
        return long(s)

    def get_resident_set_size(self):
        resident_set_size = long(0)
        try:
            f = open('/proc/%d/statm' % self.pid)
            l = f.readline()
            f.close()
            fields = l.split()
            resident_set_size = pywbem.Uint64(fields[1]) * _pagesize
        except:
            pass
        return resident_set_size

    def get_shared_mem_size(self):
        shared_mem_size = long(0)
        try:
            f = open('/proc/%d/statm' % self.pid)
            l = f.readline()
            f.close()
            fields = l.split()
            shared_mem_size = pywbem.Uint64(fields[2]) * _pagesize
        except:
            pass
        return shared_mem_size

    def get_proc_stats(self):
        return get_proc_stats(self.pid)

    def __str__(self):
        return 'LinuxProcess object. pid: %d  name: %s' \
            % (self.pid, self.get_exec_name())

    def _repr__(self):
        return self.__str__()

def _bytes2KB(v):
    kb = long(v) / 1024
    if (v % 1024) >= (1024/2):
        kb += 1
    return kb

class OMC_UnixProcessProvider(CIMProvider2):
    """Instrument the CIM class OMC_UnixProcess 

    A Unix Process
    
    """

    def fill_instance(self, model, lp, keys_only=False):
        ux = model.update
        ux(OSCreationClassName= 'OMC_OperatingSystem')
        ux(CreationClassName= 'OMC_UnixProcess')
        ux(Handle= str(lp.pid))
        ux(CSCreationClassName= 'OMC_ComputerSystem')
        ux(OSName= 'Linux')
        ux(CSName= getfqdn())
        if keys_only:
            return model
        pname = lp.get_exec_name()
        ux(Name= pname)
        ux(Caption= 'Linux Process. Name:%s PID:%d' % (pname, lp.pid))
        ux(Description= 'Linux Process. Name:%s PID:%d' % (pname, lp.pid))
        ps = lp.get_proc_stats()
        ux(ExecutionState= pywbem.Uint16(ps['state']))
        ux(ParentProcessID= str(ps['parent_process_id']))
        ux(ProcessGroupID= pywbem.Uint64(ps['process_group']))
        ux(ProcessSessionID= pywbem.Uint64(ps['session_id']))
        tty_n = ps['tty_number']
        ux(ProcessTTY=pywbem.CIMProperty('ProcessTTY', value=None, 
            type='string'))
        if tty_n > 0:
            try:
                for fd in os.listdir('/proc/%s/fd' % lp.pid):
                    slink = '/proc/%s/fd/%s' % (lp.pid, fd)
                    try:
                        if os.stat(slink).st_rdev == tty_n:
                            ux(ProcessTTY=os.readlink(slink))
                            break
                    except OSError:
                        pass
            except OSError:
                pass
        pri = ps['priority']
        if pri < 0:
            pri = 0
        ux(Priority= pywbem.Uint32(pri))
        nv = ps['nice_value']
        ux(SignedProcessNiceValue= pywbem.Sint16(nv))
        if nv < 0:
            nv = 0
        ux(ProcessNiceValue= pywbem.Uint32(nv))
        ux(CreationDate= ps['start_time'])
        ux(VirtualMemorySize= pywbem.Uint64(_bytes2KB(ps['virtual_mem_size'])))
        ux(PercentCPU= pywbem.Real32(ps['percent_cpu']))
        ux(KernelModeTime= pywbem.Uint64(ps['kernel_mode_time']))
        ux(UserModeTime= pywbem.Uint64(ps['user_mode_time']))
        ux(ModulePath=pywbem.CIMProperty('ModulePath', type='string',
            value=lp.get_module_path()))
        ux(SharedMemorySize= pywbem.Uint64(_bytes2KB(ps['shared_mem_size'])))
        ux(ResidentSetSize= pywbem.Uint64(_bytes2KB(ps['resident_set_size'])))
        cl = lp.get_command_line()
        if cl:
            ux(Parameters= cl)
        ruid = lp.get_real_uid()
        ux(RealUserID= pywbem.Uint64(ruid))
        ''' TODO where did this come from?  Not in schema. 
        try:
            ux(UserName= pwd.getpwuid(ruid)[0])
        except:
            pass
        '''
        ux(ElementName= pname)
        ux(HealthState= pywbem.Uint16(0))
        ux(OperationalStatus= [pywbem.Uint16(0)])
        return model

    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        try:
            pid = int(model['handle'])
        except:
            # Not a numeric pid
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        lp = LinuxProcess.get_proc(pid)
        if not lp:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        return self.fill_instance(model, lp)

    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)

        model.path.update({'OSCreationClassName':None,
                'CreationClassName':None,
                'Handle':None,
                'CSCreationClassName':None,
                'OSName':None,
                'CSName':None})
        for lp in LinuxProcess.procs():
            yield self.fill_instance(model, lp, keys_only)

    def set_instance(self, env, instance, modify_existing):
        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        return instance

    def delete_instance(self, env, instance_name):
        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    def cim_method_sendsignal(self, env, object_name, 
                              param_signal):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_sendsignal()' \
                % self.__class__.__name__)
        try:
            pid = int(object_name['handle'])
        except:
            # Not a numeric pid
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        lp = LinuxProcess.get_proc(pid)
        if not lp:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        ''' TODO put this back when env.get_user_name() is implemented
        user_name = env.get_user_name()
        ruid = lp.get_real_uid()
        puser_name = pwd.getpwuid(ruid)[0]
        if user_name != 'root' and user_name != puser_name:
            return ('Fail', [])
        logger = env.get_logger()
        logger.log_info('User %s sending signal %s to process %d' \
            % (user_name, str(param_signal), lp.pid))
            '''
        cc = lp.kill(param_signal)
        if cc < 0:
            return ('Fail', [])
        return ('Success', [])
        
    def cim_method_requeststatechange(self, env, object_name, 
                                      param_requestedstate,
                                      param_timeoutperiod):
        # return 'not supported'
        return (pywbem.Uint32(1), [])
        
    def cim_method_killall(self, env, object_name, 
                           param_arg):
        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_killall()' \
                % self.__class__.__name__)
        user_name = env.get_user_name()
        if (user_name != 'root'):
            return ('Fail', [])
        for lp in LinuxProcess.procs():
            if lp.get_exec_name() == param_arg:
                lp.kill(signal.SIGTERM)
        return ('Success', [])
        
    class Values(object):
        class Status(object):
            OK = 'OK'
            Error = 'Error'
            Degraded = 'Degraded'
            Unknown = 'Unknown'
            Pred_Fail = 'Pred Fail'
            Starting = 'Starting'
            Stopping = 'Stopping'
            Service = 'Service'
            Stressed = 'Stressed'
            NonRecover = 'NonRecover'
            No_Contact = 'No Contact'
            Lost_Comm = 'Lost Comm'
            Stopped = 'Stopped'

        class RequestedState(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            No_Change = pywbem.Uint16(5)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)
            Not_Applicable = pywbem.Uint16(12)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class HealthState(object):
            Unknown = pywbem.Uint16(0)
            OK = pywbem.Uint16(5)
            Degraded_Warning = pywbem.Uint16(10)
            Minor_failure = pywbem.Uint16(15)
            Major_failure = pywbem.Uint16(20)
            Critical_failure = pywbem.Uint16(25)
            Non_recoverable_error = pywbem.Uint16(30)
            # DMTF_Reserved = ..

        class ExecutionState(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Ready = pywbem.Uint16(2)
            Running = pywbem.Uint16(3)
            Blocked = pywbem.Uint16(4)
            Suspended_Blocked = pywbem.Uint16(5)
            Suspended_Ready = pywbem.Uint16(6)
            Terminated = pywbem.Uint16(7)
            Stopped = pywbem.Uint16(8)
            Growing = pywbem.Uint16(9)
            Ready_But_Relinquished_Processor = pywbem.Uint16(10)
            Hung = pywbem.Uint16(11)

        class EnabledDefault(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            No_Default = pywbem.Uint16(7)
            Quiesce = pywbem.Uint16(9)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class EnabledState(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shutting_Down = pywbem.Uint16(4)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            In_Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Starting = pywbem.Uint16(10)
            # DMTF_Reserved = 11..32767
            # Vendor_Reserved = 32768..65535

        class OperationalStatus(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            OK = pywbem.Uint16(2)
            Degraded = pywbem.Uint16(3)
            Stressed = pywbem.Uint16(4)
            Predictive_Failure = pywbem.Uint16(5)
            Error = pywbem.Uint16(6)
            Non_Recoverable_Error = pywbem.Uint16(7)
            Starting = pywbem.Uint16(8)
            Stopping = pywbem.Uint16(9)
            Stopped = pywbem.Uint16(10)
            In_Service = pywbem.Uint16(11)
            No_Contact = pywbem.Uint16(12)
            Lost_Communication = pywbem.Uint16(13)
            Aborted = pywbem.Uint16(14)
            Dormant = pywbem.Uint16(15)
            Supporting_Entity_in_Error = pywbem.Uint16(16)
            Completed = pywbem.Uint16(17)
            Power_Mode = pywbem.Uint16(18)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class RequestStateChange(object):
            Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown_or_Unspecified_Error = pywbem.Uint32(2)
            Cannot_complete_within_Timeout_Period = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Invalid_State_Transition = pywbem.Uint32(4097)
            Use_of_Timeout_Parameter_Not_Supported = pywbem.Uint32(4098)
            Busy = pywbem.Uint32(4099)
            # Method_Reserved = 4100..32767
            # Vendor_Specific = 32768..65535
            class RequestedState(object):
                Enabled = pywbem.Uint16(2)
                Disabled = pywbem.Uint16(3)
                Shut_Down = pywbem.Uint16(4)
                Offline = pywbem.Uint16(6)
                Test = pywbem.Uint16(7)
                Defer = pywbem.Uint16(8)
                Quiesce = pywbem.Uint16(9)
                Reboot = pywbem.Uint16(10)
                Reset = pywbem.Uint16(11)
                # DMTF_Reserved = ..
                # Vendor_Reserved = 32768..65535

## end of class OMC_UnixProcessProvider

class OMC_OSProcessProvider(CIMProvider2):
    """Instrument the CIM class OMC_OSProcess 

    The relationship between the OperatingSystem and Process(es) running in the
    context of this OperatingSystem.
    """

    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        pc = model['PartComponent']
        gc = model['GroupComponent']
        lp = LinuxProcess.get_proc(pc['handle'])
        if not lp:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        return model

    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
        model.path.update(PartComponent=None, GroupComponent=None)
        for lp in LinuxProcess.procs():
            model['PartComponent'] = lp.get_instance_name(
                    model.path.namespace)
            model['GroupComponent'] = get_os_path(model.path.namespace)
            yield model

    def set_instance(self, env, instance, modify_existing):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement

    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    def references(self, env, object_name, model, 
                   result_class_name, role, result_role, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.references()' \
                % self.__class__.__name__)
        model.path.update({'GroupComponent':None,'PartComponent':None})
        is_subclass = env.get_cimom_handle().is_subclass
        if object_name.classname.lower() == 'omc_operatingsystem':
            if role and role.lower() != 'groupcomponent':
                return
            if result_role and result_role.lower() != 'partcomponent':
                return
            if result_class_name:
                if not is_subclass(object_name.namespace, sub='omc_unixprocess',
                        super=result_class_name):
                    return
            for lp in LinuxProcess.procs():
                model['GroupComponent'] = object_name
                model['PartComponent'] = lp.get_instance_name(
                        object_name.namespace)
                yield model
            return

        if object_name.classname.lower() == 'omc_unixprocess':
            if not 'handle' in object_name:
                return
            if role and role.lower() != 'partcomponent':
                return
            if result_role and result_role.lower() != 'groupcomponent':
                return
            if result_class_name:
                if not is_subclass(object_name.namespace, 
                        sub='omc_operatingsystem',
                        super=result_class_name):
                    return
            lp = LinuxProcess.get_proc(object_name['handle'])
            if not lp:
                return
            model['PartComponent'] = object_name
            model['GroupComponent'] = get_os_path(object_name.namespace)
            yield model
## end of class OMC_OSProcessProvider

class OMC_ProcessExecutableProvider(CIMProvider2):
    """Instrument the CIM class OMC_ProcessExecutable 

    A link between a Process and a DataFile indicating that the File
    participates in the execution of the Process.
    """

    def get_instance(self, env, model):
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        dn = model['Dependent']
        lp = LinuxProcess.get_proc(dn['handle'])
        if not lp:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        an = model['Antecedent']
        fname = an['name']
        if fname != lp.get_module_path():
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        return model

    def enum_instances(self, env, model, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
        for lp in LinuxProcess.procs():
            model['Dependent'] = lp.get_instance_name(model.path.namespace)
            fname = lp.get_module_path()
            if fname:
                model['Antecedent'] = OMC_LogicalFile.get_file_path(fname, model.path.namespace)
                yield model

    def set_instance(self, env, instance, modify_existing):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) 

    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) 
        
    def references(self, env, object_name, model, 
                   result_class_name, role, result_role, keys_only):
        logger = env.get_logger()
        logger.log_debug('Entering %s.references()' \
                % self.__class__.__name__)

        model.path.update({'Dependent':None, 'Antecedent':None})
        is_subclass = env.get_cimom_handle().is_subclass
        if object_name.classname.lower() == 'omc_unixprocess':
            if not 'handle' in object_name:
                return
            lp = LinuxProcess.get_proc(object_name['handle'])
            if not lp:
                return
            if role and role.lower() != 'dependent':
                return
            if result_role and result_role.lower() != 'antecedent':
                return
            if result_class_name:
                if is_subclass(object_name.namespace, 
                        sub=result_class_name, super='omc_datafile'):
                    pass
                elif not is_subclass(object_name.namespace, sub='omc_datafile',
                        super=result_class_name):
                    return
            model['Dependent'] = object_name
            fname = '/proc/%d/exe' % lp.pid
            if os.path.exists(fname):
                fname = os.readlink(fname)
                fpath = OMC_LogicalFile.get_file_path(fname)
                fpath.namespace = object_name.namespace
                model['Antecedent'] = fpath
                yield model
            return

        if is_subclass(object_name.namespace, 
                       sub=object_name.classname, super='omc_datafile'):
            if role and role.lower() != 'antecedent':
                return
            if result_role and result_role.lower() != 'dependent':
                return
            if result_class_name:
                if not is_subclass(object_name.namespace, 
                               sub='omc_unixprocess', super=result_class_name):
                    return
            model['Antecedent'] = object_name
            fname = object_name['name']
            if not os.path.exists(fname):
                return
            for lp in LinuxProcess.procs():
                if lp.get_module_path() == fname:
                    model['Dependent'] = lp.get_instance_name(
                            object_name.namespace)
                    yield model

def get_providers(env):
    return {'OMC_UnixProcess': OMC_UnixProcessProvider(), 
            'OMC_ProcessExecutable': OMC_ProcessExecutableProvider(),
            'OMC_OSProcess': OMC_OSProcessProvider()}

